@@ -30,7 +30,7 @@ for molecule in data:
     # split the structures
     s1 = ss.find_connected(0)
     s2 = ss.find_connected(-1)
-    assert(len(ss) == len(s1) + len(s2))
+    assert len(ss) == len(s1) + len(s2)
     if xc == 'TS09' or xc == 'TPSS' or xc == 'M06-L' or xc == 'dftd4':
         c = GPAW(xc='PBE', h=h, nbands=-6, occupations=FermiDirac(width=0.1))
     else:
