@@ -16,7 +16,7 @@ def link_shared_object (self,
         (output_dir, output_filename) = os.path.split(output_filename)
     output_fullname = os.path.join(output_dir, output_filename)
     output_fullname = os.path.abspath(output_fullname)
-    linkline = "%s %s" % (output_filename[:-2], output_fullname)
+    linkline = f"{output_filename[:-2]} {output_fullname}"
     for l in library_dirs:
         linkline += " -L" + l
     for l in libraries:
