@@ -13,7 +13,7 @@ direct_gap = np.zeros(4)
 for j, k in enumerate([6, 8, 10, 12]):
     for i, ecut in enumerate([100, 200, 300, 400]):
         fil = pickle.load(
-            paropen(f'C-g0w0_k{k}_ecut{ecut}_results.pckl', 'rb'))
+            paropen(f'C-g0w0_k{k}_ecut{ecut}_results_GW.pckl', 'rb'))
         direct_gap[i] = fil['qp'][0, 0, 1] - fil['qp'][0, 0, 0]
     plt.plot(ecuts, direct_gap, color[j],
              label=f'({k}x{k}x{k}) k-points')
