@@ -10,9 +10,6 @@ def workflow():
         with run(script='C_frequency_conv_plot.py'):
             run(script='C_equal_test.py', deps=[r])
 
-    with run(script='BN_GW0.py', tmax='1h'):
-        run(script='BN_GW0_plot.py')
-
     with run(script='MoS2_gs_GW.py', tmax='2h'):
         with run(script='MoS2_GWG.py', cores=8, tmax='20m'):
             run(script='MoS2_bs_plot.py')
