@@ -9,7 +9,9 @@ atoms.center(vacuum=8)
 calc = GPAW(h=0.2,
             nbands=14,
             xc='LDA',
-            poissonsolver={'remove_moment': 1 + 3 + 5},
+            poissonsolver={'name': 'MomentCorrectionPoissonSolver',
+                           'poissonsolver': 'fast',
+                           'moment_corrections': 1 + 3 + 5},
             convergence={'bands': 'occupied'},
             txt='gs.out')
 atoms.calc = calc
