@@ -2,6 +2,7 @@ import numbers
 from math import pi, factorial as fac
 
 import numpy as np
+from scipy.interpolate import make_interp_spline, splder
 
 from gpaw.spline import Spline
 from gpaw.utilities import hartree, divrl
@@ -150,6 +151,23 @@ class RadialGridDescriptor:
         vr_g[:] *= self.r_g[:]**0.5
         return vr_g
 
+    def derivative_spline(self, n_g, dndr_g=None, *, degree=5):
+        """Spline-based derivative of radial function."""
+        if dndr_g is None:
+            dndr_g = self.empty()
+
+        # Boundary conditions
+        assert degree in [3, 5]
+        if degree == 5:
+            bc_type = ([(2, 0.0), (4, 0.0)], [(2, 0.0), (4, 0.0)])
+        elif degree == 3:
+            bc_type = ([(2, 0.0)], [(2, 0.0)])
+
+        s = make_interp_spline(self.r_g, n_g, k=degree, bc_type=bc_type)
+        s = splder(s)
+        dndr_g[:] = s(self.r_g)
+        return dndr_g
+
     def derivative(self, n_g, dndr_g=None):
         """Finite-difference derivative of radial function."""
         if dndr_g is None:
