@@ -1,17 +1,19 @@
 
+from itertools import product
+
 import numpy as np
-
-from scipy.spatial import Delaunay, Voronoi, ConvexHull
-from scipy.spatial.qhull import QhullError
-
 from ase.dft.kpoints import monkhorst_pack
+from scipy.spatial import ConvexHull, Delaunay, Voronoi
+try:
+    from scipy.spatial import QhullError
+except ImportError:  # scipy < 1.8
+    from scipy.spatial.qhull import QhullError
 
 import gpaw.mpi as mpi
 from gpaw import GPAW, restart
-from gpaw.symmetry import Symmetry, aglomerate_points
-from gpaw.kpt_descriptor import to1bz, kpts2sizeandoffsets
-from itertools import product
+from gpaw.kpt_descriptor import kpts2sizeandoffsets, to1bz
 from gpaw.mpi import world
+from gpaw.symmetry import Symmetry, aglomerate_points
 
 
 def get_lattice_symmetry(cell_cv, tolerance=1e-7):
@@ -32,8 +34,7 @@ def get_lattice_symmetry(cell_cv, tolerance=1e-7):
     return latsym
 
 
-def find_high_symmetry_monkhorst_pack(calc, density,
-                                      pbc=None):
+def find_high_symmetry_monkhorst_pack(calc, density):
     """Make high symmetry Monkhorst Pack k-point grid.
 
     Searches for and returns a Monkhorst Pack grid which
@@ -47,9 +48,6 @@ def find_high_symmetry_monkhorst_pack(calc, density,
         The path to a calculator object.
     density : float
         The required minimum density of the Monkhorst Pack grid.
-    pbc : Boolean list/ndarray of shape (3,) or None
-        List indicating periodic directions. If None then
-        pbc = [True] * 3.
 
     Returns
     -------
@@ -58,12 +56,8 @@ def find_high_symmetry_monkhorst_pack(calc, density,
 
     """
 
-    if pbc is None:
-        pbc = np.array([True, True, True])
-    else:
-        pbc = np.array(pbc)
-
     atoms, calc = restart(calc, txt=None)
+    pbc = atoms.pbc
     minsize, offset = kpts2sizeandoffsets(density=density, even=True,
                                           gamma=True, atoms=atoms)
 
