@@ -20,7 +20,7 @@ class CLICommand:
         parser.add_argument('arguments', metavar='ARG',
                             help='Arguments passed to program in '
                             'sys.argv[1:].',
-                            nargs=-1)
+                            nargs='*')
 
     @staticmethod
     def run(args):
