@@ -1,20 +1,21 @@
 """BLACS distributed matrix object."""
 from __future__ import annotations
-from typing import Dict, Tuple
-import numpy as np
-import scipy.linalg as linalg
+
+from typing import Dict, Tuple, Union
 
 import _gpaw
-from gpaw import debug
-from gpaw.mpi import serial_comm, _Communicator
 import gpaw.utilities.blas as blas
-
+import numpy as np
+import scipy.linalg as linalg
+from gpaw import debug
+from gpaw.mpi import MPIComm, _Communicator, serial_comm
+from gpaw.typing import Array1D, ArrayLike1D, ArrayLike2D
 
 _global_blacs_context_store: Dict[Tuple[_Communicator, int, int], int] = {}
 
 
-def suggest_blocking(N, ncpus):
-    """Suggest blocking of NxN matrix.
+def suggest_blocking(N: int, ncpus: int) -> tuple[int, int, int]:
+    """Suggest blocking of ``NxN`` matrix.
 
     Returns rows, columns, blocksize tuple.
 
@@ -50,30 +51,42 @@ def suggest_blocking(N, ncpus):
 
 
 class Matrix:
-    def __init__(self, M, N, dtype=None, data=None, dist=None):
+    def __init__(self,
+                 M: int,
+                 N: int,
+                 dtype=None,
+                 data: ArrayLike2D = None,
+                 dist: Union[MatrixDistribution, tuple[int, ...]] = None):
         """Matrix object.
 
-        M: int
+        Parameters
+        ----------
+        M:
             Rows.
-        N: int
+        N:
             Columns.
-        dtype: type
+        dtype:
             Data type (float or complex).
-        dist: tuple or None
-            BLACS distribution given as (communicator, rows, colums, blocksize)
+        dist:
+            BLACS distribution given as
+            (communicator, rows, columns, blocksize)
             tuple.  Default is None meaning no distribution.
-        data: ndarray or None.
-            Numpy ndarray to use for starage.  By default, a new ndarray
+        data:
+            Numpy ndarray to use for storage.  By default, a new ndarray
             will be allocated.
             """
         self.shape = (M, N)
 
+        if data is not None:
+            data = np.asarray(data)
+
         if dtype is None:
             if data is None:
                 dtype = float
             else:
                 dtype = data.dtype
         self.dtype = np.dtype(dtype)
+        assert dtype == float or dtype == complex, dtype
 
         dist = dist or ()
         if isinstance(dist, tuple):
@@ -89,7 +102,7 @@ class Matrix:
         dist = str(self.dist).split('(')[1]
         return 'Matrix({}: {}'.format(self.dtype.name, dist)
 
-    def new(self, dist='inherit', data=None):
+    def new(self, dist='inherit', data=None) -> Matrix:
         """Create new matrix of same shape and dtype.
 
         Default is to use same BLACS distribution.  Use dist to use another
@@ -100,11 +113,23 @@ class Matrix:
                       dist=self.dist if dist == 'inherit' else dist,
                       data=data)
 
+    def copy(self) -> Matrix:
+        """Create a copy."""
+        M = self.new()
+        M.data[:] = self.data
+        return M
+
     def __setitem__(self, item, value):
         assert item == slice(None)
         assert isinstance(value, Matrix)
         self.data[:] = value.data
 
+    def __iadd__(self, other):
+        if isinstance(other, Matrix):
+            other = other.data
+        self.data += other
+        return self
+
     def multiply(self,
                  other,
                  alpha=1.0,
@@ -113,6 +138,7 @@ class Matrix:
                  out=None,
                  beta=0.0,
                  symmetric=False) -> Matrix:
+        """BLAS matrix-multiplication with other matrix."""
         if not isinstance(other, Matrix):
             other = other.matrix
         A = self
@@ -127,7 +153,7 @@ class Matrix:
         elif not isinstance(out, Matrix):
             out = out.matrix
 
-        if dist.comm.size > 1:
+        if 0:  # dist.comm.size > 1:
             # Special cases that don't need scalapack - most likely also
             # faster:
             if alpha == 1.0 and opa == 'N' and opb == 'N':
@@ -141,7 +167,7 @@ class Matrix:
         dist.multiply(alpha, A, opa, B, opb, beta, out, symmetric)
         return out
 
-    def redist(self, other):
+    def redist(self, other: Matrix) -> None:
         """Redistribute to other BLACS layout."""
         if self is other:
             return
@@ -155,9 +181,10 @@ class Matrix:
 
         if n2 == 1 and d1.blocksize is None:
             assert d2.blocksize is None
+            assert d1.columns == 1
             comm = d1.comm
             if comm.rank == 0:
-                M = len(self)
+                M = self.shape[0]
                 m = (M + comm.size - 1) // comm.size
                 other.data[:m] = self.data
                 for r in range(1, comm.size):
@@ -170,9 +197,10 @@ class Matrix:
 
         if n1 == 1 and d2.blocksize is None:
             assert d1.blocksize is None
+            assert d1.columns == 1
             comm = d1.comm
             if comm.rank == 0:
-                M = len(self)
+                M = self.shape[0]
                 m = (M + comm.size - 1) // comm.size
                 other.data[:] = self.data[:m]
                 for r in range(1, comm.size):
@@ -199,16 +227,56 @@ class Matrix:
                 ctx = d2.desc[1]
             redist(d1, self.data, d2, other.data, ctx)
 
-    def invcholesky(self):
-        """Inverse of Cholesky decomposition.
+    def gather(self, root: int = 0) -> Matrix:
+        """Gather the Matrix on the root rank.
 
-        Only the lower part is used.
+        Returns a new Matrix distributed so that all data is on the root rank
         """
+        assert root == 0
         if self.dist.comm.size > 1:
             S = self.new(dist=(self.dist.comm, 1, 1))
             self.redist(S)
         else:
             S = self
+
+        return S
+
+    def inv(self, uplo='L'):
+        """Inplace inversion."""
+        assert uplo == 'L'
+        M, N = self.shape
+        assert M == N
+        dist = self.dist
+        if dist.comm.size == 1:
+            self.tril2full()
+            self.data[:] = linalg.inv(self.data,
+                                      overwrite_a=True,
+                                      check_finite=debug)
+            return
+        bc, br = dist.desc[4:6]
+        assert bc == br
+        info = _gpaw.scalapack_inverse(self.data, dist.desc, 'U')
+        if info != 0:
+            raise ValueError(f'scalapack_inverse error: {info}')
+
+    def invcholesky(self) -> None:
+        """Inverse of Cholesky decomposition.
+
+        Returns a lower triangle matrix `L` where:::
+
+             †
+          LSL = 1
+
+        Only the lower part of `S` is used.
+
+        >>> S = Matrix(2, 2, data=[[1.0, np.nan],
+        ...                        [0.1, 1.0]])
+        >>> S.invcholesky()
+        >>> S.data
+        array([[ 1.        , -0.        ],
+               [-0.10050378,  1.00503782]])
+        """
+        S = self.gather()
         if self.dist.comm.rank == 0:
             if debug:
                 S.data[np.triu_indices(S.shape[0], 1)] = 42.0
@@ -222,16 +290,26 @@ class Matrix:
         if S is not self:
             S.redist(self)
 
-    def eigh(self, cc=False, scalapack=(None, 1, 1, None)):
+    def eigh(self,
+             S=None,
+             *,
+             cc=False,
+             scalapack=(None, 1, 1, None),
+             limit: int = None) -> Array1D:
         """Calculate eigenvectors and eigenvalues.
 
         Matrix must be symmetric/hermitian and stored in lower half.
+        If ``S`` is given, solve a generalized eigenvalue problem.
 
+        Parameters
+        ----------
         cc: bool
             Complex conjugate matrix before finding eigenvalues.
         scalapack: tuple
             BLACS distribution for ScaLapack to use.  Default is to do serial
             diagonalization.
+        limit:
+            Number of eigenvector and values to find.  Defaults to all.
         """
         slcomm, rows, columns, blocksize = scalapack
 
@@ -245,6 +323,10 @@ class Matrix:
         if redist:
             H = self.new(dist=dist)
             self.redist(H)
+            if S is not None:
+                S0 = S
+                S = S0.new(dist=dist)
+                self.redist(S)
         else:
             assert self.dist.comm.size == slcomm.size
             H = self
@@ -257,14 +339,30 @@ class Matrix:
                     np.negative(H.data.imag, H.data.imag)
                 if debug:
                     H.data[np.triu_indices(H.shape[0], 1)] = 42.0
-                eps[:], H.data.T[:] = linalg.eigh(H.data,
-                                                  lower=True,  # ???
-                                                  overwrite_a=True,
-                                                  check_finite=debug)
+                if S is None:
+                    eps[:], H.data.T[:] = linalg.eigh(
+                        H.data,
+                        lower=True,
+                        overwrite_a=True,
+                        check_finite=debug,
+                        driver='evx' if H.data.size == 1 else 'evd')
+                else:
+                    eps, evecs = linalg.eigh(
+                        H.data,
+                        S.data,
+                        lower=True,
+                        overwrite_a=True,
+                        overwrite_b=True,
+                        check_finite=debug,
+                        subset_by_index=(0, limit - 1) if limit else None,
+                        driver='evx' if H.data.size == 1 else 'evd')
+                    limit = limit or len(eps)
+                    H.data.T[:, :limit] = evecs
             self.dist.comm.broadcast(eps, 0)
         else:
             if slcomm.rank < rows * columns:
                 assert cc
+                assert S is None
                 array = H.data.copy()
                 info = _gpaw.scalapack_diagonalize_dc(array, H.dist.desc, 'U',
                                                       H.data, eps)
@@ -281,11 +379,114 @@ class Matrix:
 
         return eps
 
-    def complex_conjugate(self):
+    def eighg(self, L: Matrix, comm2: MPIComm = serial_comm) -> Array1D:
+        """Solve generalized eigenvalue problem.
+
+        :::
+
+          HC = SCΛ,
+
+        where `L` is a lower triangle matrix such that:::
+
+             †
+          LSL = 1.
+
+        The solution has these three steps:::
+
+           ~      †   ~~   ~         †~
+           H = LHL ,  HC = CΛ,  C = L C.
+        """
+        M, N = self.shape
+        assert M == N
+        comm = self.dist.comm
+
+        if comm2.rank == 0:
+            if comm.size == 1:
+                H = self
+            else:
+                H = self.new(dist=(comm,))
+                self.redist(H)
+            if comm.rank == 0:
+                tmp_MM = np.empty_like(H.data)
+                L_MM = L.data
+                blas.mmm(1.0, L_MM, 'N', H.data, 'N', 0.0, tmp_MM)
+                blas.r2k(0.5, tmp_MM, L_MM, 0.0, H.data)
+                # Ht_MM = L_MM @ self.data @ L_MM.conj().T
+                eig_n, Ct_Mn = linalg.eigh(
+                    H.data,
+                    overwrite_a=True,
+                    check_finite=debug,
+                    driver='evx' if M == 1 else 'evd')
+                assert Ct_Mn.flags.f_contiguous
+                blas.mmm(1.0, L_MM, 'C', Ct_Mn.T, 'T', 0.0, H.data)
+                # self.data[:] = L_MM.T.conj() @ Ct_Mn
+            else:
+                eig_n = np.empty(M)
+
+            if comm.size > 1:
+                H.redist(self)
+                comm.broadcast(eig_n, 0)
+
+        if comm2.rank > 0:
+            eig_n = np.empty(M)
+        comm2.broadcast(eig_n, 0)
+        comm2.broadcast(self.data, 0)
+
+        return eig_n
+
+    def complex_conjugate(self) -> None:
         """Inplace complex conjugation."""
         if self.dtype == complex:
             np.negative(self.data.imag, self.data.imag)
 
+    def add_hermitian_conjugate(self, scale: float = 1.0) -> None:
+        """Add hermitian conjugate to myself."""
+        if self.dist.comm.size == 1:
+            self.data *= 0.5
+            self.data += self.data.conj().T
+            return
+        tmp = self.copy()
+        _gpaw.pblas_tran(*self.shape, scale, tmp.data, scale, self.data,
+                         self.dist.desc, self.dist.desc, True)
+
+    def tril2full(self) -> None:
+        """Fill in upper triangle from lower triangle.
+
+        For a real matrix::
+
+          a ? ?    a b d
+          b c ? -> b c e
+          d e f    d e f
+
+        For a complex matrix, the complex conjugate of the lower part will
+        be inserted into the upper part.
+        """
+        M, N = self.shape
+        assert M == N
+
+        dist = self.dist
+
+        if dist.comm.size == 1 or dist.rows == 1 and dist.columns == 1:
+            if dist.comm.rank == 0:
+                u = np.triu_indices(M, 1)
+                self.data[u] = self.data.T[u].conj()
+            return
+
+        desc = dist.desc
+        _gpaw.scalapack_set(self.data, desc, 0.0, 0.0, 'L', M - 1, M - 1, 2, 1)
+        buf = self.data.copy()
+        # Set diagonal to zero in the copy:
+        _gpaw.scalapack_set(buf, desc, 0.0, 0.0, 'L', M, M, 1, 1)
+        # Now transpose tmp_mm adding the result to the original matrix:
+        _gpaw.pblas_tran(M, M, 1.0, buf, 1.0, self.data, desc, desc, True)
+
+    def add_to_diagonal(self, d: ArrayLike1D | float) -> None:
+        """Add list of numbers or single number to diagonal of matrix."""
+        n1, n2 = self.dist.my_row_range()
+        M, N = self.shape
+        assert M == N
+        self.data.ravel()[n1::N + 1] += d
+
 
 def _matrix(M):
     """Dig out Matrix object from wrapper(s)."""
@@ -295,9 +496,36 @@ def _matrix(M):
 
 
 class MatrixDistribution:
+    comm: MPIComm
+    rows: int
+    columns: int
+    blocksize: int | None  # None means everything on rank=0
+    shape: tuple[int, int]
+    desc: Array1D
+
     def matrix(self, dtype=None, data=None):
         return Matrix(*self.full_shape, dtype=dtype, data=data, dist=self)
 
+    def multiply(self, alpha, a, opa, b, opb, beta, c, symmetric):
+        raise NotImplementedError
+
+    def my_row_range(self) -> tuple[int, int]:
+        """Return indices for range of my rows.
+
+        >>> Matrix(2, 2).dist.my_row_range()
+        (0, 2)
+        """
+        ok = (self.rows == self.comm.size and
+              self.columns == 1 and
+              self.blocksize is None)
+        if not ok:
+            raise ValueError(f'Can not create slice of distribution: {self}')
+        M = self.shape[0]
+        b = (M + self.rows - 1) // self.rows
+        n1 = self.comm.rank * b
+        n2 = min(n1 + b, M)
+        return n1, n2
+
 
 class NoDistribution(MatrixDistribution):
     comm = serial_comm
@@ -428,7 +656,12 @@ def redist(dist1, M1, dist2, M2, context):
                            context, 'G')
 
 
-def create_distribution(M, N, comm=None, r=1, c=1, b=None):
+def create_distribution(M: int,
+                        N: int,
+                        comm: MPIComm = None,
+                        r: int = 1,
+                        c: int = 1,
+                        b: int = None) -> MatrixDistribution:
     if comm is None or comm.size == 1:
         assert r == 1 and abs(c) == 1 or c == 1 and abs(r) == 1
         return NoDistribution(M, N)
@@ -444,7 +677,7 @@ def fastmmm(m1, m2, m3, beta):
 
     buf1 = m2.data
 
-    N = len(m1)
+    N = m1.shape[0]
     n = (N + comm.size - 1) // comm.size
 
     for r in range(comm.size):
