@@ -1,5 +1,6 @@
 import scipy.sparse.linalg as sla
 
+
 class ScipyLinearSolver:
     """Wrapper class for the linear solvers in ``scipy.sparse.linalg``.
 
@@ -15,13 +16,13 @@ class ScipyLinearSolver:
     """
 
     # Supported solvers
-    solvers = {'cg':       sla.cg,       # symmetric positive definite
-               'minres':   sla.minres,   # symmetric indefinite
-               'gmres':    sla.gmres,    # non-symmetric
-               'bicg':     sla.bicg,     # non-symmetric
-               'cgs':      sla.cgs,      # similar to bicg
-               'bicgstab': sla.bicgstab, #
-               'qmr':      sla.qmr,      #
+    solvers = {'cg': sla.cg,  # symmetric positive definite
+               'minres': sla.minres,  # symmetric indefinite
+               'gmres': sla.gmres,  # non-symmetric
+               'bicg': sla.bicg,  # non-symmetric
+               'cgs': sla.cgs,  # similar to bicg
+               'bicgstab': sla.bicgstab,
+               'qmr': sla.qmr,
                # 'lgmres': sla.lgmres, # scipy v. 0.8.0
                }
 
