@@ -12,25 +12,24 @@ See:
     https://doi.org/10.1103/PhysRevB.62.6158
 
 """
-from typing import List
 from math import pi
+from typing import List
 
+import ase.units as units
 import numpy as np
 from scipy.integrate import simps
-import ase.units as units
 
-from gpaw import GPAW
-from gpaw.atom.radialgd import RadialGridDescriptor
+from gpaw.calculator import GPAW
 from gpaw.atom.aeatom import Channel
 from gpaw.atom.configurations import configurations
-from gpaw.setup import Setup
-from gpaw.grid_descriptor import GridDescriptor
-from gpaw.wavefunctions.pw import PWDescriptor
-from gpaw.utilities import unpack2
+from gpaw.atom.radialgd import RadialGridDescriptor
 from gpaw.gaunt import gaunt
+from gpaw.grid_descriptor import GridDescriptor
+from gpaw.setup import Setup
+from gpaw.typing import Array1D, Array2D, Array3D
+from gpaw.utilities import unpack2
+from gpaw.pw.descriptor import PWDescriptor
 from gpaw.xc.functional import XCFunctional
-from gpaw.hints import Array1D, Array2D, Array3D
-
 
 # Fine-structure constant: (~1/137)
 alpha = 0.5 * units._mu0 * units._c * units._e**2 / units._hplanck
@@ -59,8 +58,7 @@ def hyperfine_parameters(calc: GPAW,
         \int \frac{3 r_i r_j - \delta_{ij} r^2}{r^5}
         \rho_s(\mathbf{r}) d\mathbf{r}.
 
-    Remember to multiply each tensor by the g-factors of the nuclei
-    and divide by the total electron spin.
+    Remember to multiply each tensor by the g-factors of the nuclei.
 
     Use ``exclude_core=True`` to exclude contribution from "frozen" core.
     """
@@ -96,7 +94,8 @@ def smooth_part(spin_density_R: Array3D,
     pd = PWDescriptor(ecut, gd)
     spin_density_G = pd.fft(spin_density_R)
     G_Gv = pd.get_reciprocal_vectors()
-    eiGR_aG = np.exp(-1j * spos_ac.dot(gd.cell_cv).dot(G_Gv.T))
+    # eiGR_aG = np.exp(-1j * spos_ac.dot(gd.cell_cv).dot(G_Gv.T))
+    eiGR_aG = np.exp(-1j * spos_ac @ gd.cell_cv @ G_Gv.T)
 
     # Isotropic term:
     W1_a = pd.integrate(spin_density_G, eiGR_aG) / gd.dv * (2 / 3)
@@ -181,7 +180,8 @@ def paw_correction(density_sii: Array3D,
     dn2_mg -= np.einsum('mab, ag, bg -> mg', D2_mjj, phit_jg, phit_jg)
     A_m = dn2_mg[:, 1:].dot(rgd.dr_g[1:] / rgd.r_g[1:]) / 5
     A_m *= Y2_m
-    W_vv = Y2_mvv.T.dot(A_m)
+    # W_vv: Array2D = Y2_mvv.T.dot(A_m)  # type: ignore
+    W_vv = Y2_mvv.T @ A_m
     W = np.trace(W_vv) / 3
     for v in range(3):
         W_vv[v, v] -= W
@@ -211,7 +211,7 @@ def expand(D_ii: Array2D,
     return D_mjj
 
 
-def delta(r: float, rT: float) -> float:
+def delta(r: Array1D, rT: float) -> Array1D:
     """Extended delta function."""
     return 2 / rT / (1 + 2 * r / rT)**2
 
@@ -229,7 +229,7 @@ def integrate(n0_g: Array1D,
     r_j = np.linspace(0, r4, n)
 
     b_i = np.arange(3, -1, -1) + 1 - beta
-    d0 = delta(0, rT)
+    d0 = 2 / rT  # delta(0, rT)
     d1 = -8 / rT**2
     n0 = a_i.dot(d0 * r4**b_i / b_i + d1 * r4**(b_i + 1) / (b_i + 1))
 
@@ -329,10 +329,7 @@ def main(argv: List[str] = None) -> None:
     add('-d', '--diagonalize', action='store_true',
         help='Show eigenvalues of tensor.')
 
-    if hasattr(parser, 'parse_intermixed_args'):
-        args = parser.parse_intermixed_args(argv)
-    else:
-        args = parser.parse_args(argv)
+    args = parser.parse_intermixed_args(argv)
 
     calc = GPAW(args.file)
     atoms = calc.get_atoms()
@@ -340,7 +337,6 @@ def main(argv: List[str] = None) -> None:
     symbols = atoms.symbols
     magmoms = atoms.get_magnetic_moments()
     total_magmom = atoms.get_magnetic_moment()
-    assert total_magmom != 0.0
 
     g_factors = {symbol: ratio * 1e6 * 4 * pi * units._mp / units._e
                  for symbol, (n, ratio) in gyromagnetic_ratios.items()}
@@ -374,9 +370,9 @@ def main(argv: List[str] = None) -> None:
         magmom = magmoms[a]
         g_factor = g_factors.get(symbol, 1.0)
         used[symbol] = g_factor
-        A_vv *= g_factor / total_magmom * scale
+        A_vv *= g_factor * scale
         if args.diagonalize:
-            numbers = np.linalg.eigvalsh(A_vv)
+            numbers = np.linalg.eigvalsh(A_vv).tolist()
         else:
             numbers = [A_vv[0, 0], A_vv[1, 1], A_vv[2, 2],
                        A_vv[1, 2], A_vv[0, 2], A_vv[0, 1]]
