@@ -11,7 +11,6 @@ from ase.utils.timing import Timer
 import _gpaw
 import gpaw.mpi as mpi
 from gpaw.xc import XC
-from gpaw.wavefunctions.fd import FDWaveFunctions
 from gpaw.lrtddft.excitation import Excitation, ExcitationList, get_filehandle
 from gpaw.lrtddft.kssingle import KSSingles
 from gpaw.lrtddft.omega_matrix import OmegaMatrix
@@ -86,15 +85,9 @@ class LrTDDFT(ExcitationList):
 
         if calculator is not None and calculator.initialized:
             # XXXX not ready for k-points
-            assert(len(calculator.wfs.kd.ibzk_kc) == 1)
-            if not isinstance(calculator.wfs, FDWaveFunctions):
-                raise RuntimeError(
-                    'Linear response TDDFT supported only in real space mode')
-            if calculator.wfs.kd.comm.size > 1:
-                err_txt = 'Spin parallelization with Linear response '
-                err_txt += "TDDFT. Use parallel={'domain': world.size} "
-                err_txt += 'calculator parameter.'
-                raise NotImplementedError(err_txt)
+            assert len(calculator.wfs.kd.bzk_kc) == 1
+            if calculator.wfs.mode not in ['fd', 'lcao']:
+                raise RuntimeError('LrTDDFT supports only fd and lcao modes')
             if calculator.parameters.mode != 'lcao':
                 calculator.converge_wave_functions()
             if calculator.density.nct_G is None:
@@ -414,7 +407,10 @@ class LrTDDFT(ExcitationList):
         return list.__len__(self)
 
     def __del__(self):
-        self.timer.write(self.log.fd)
+        try:
+            self.timer.write(self.log.fd)
+        except AttributeError:
+            pass
 
 
 def d2Excdnsdnt(dup, ddn):
