@@ -1,6 +1,7 @@
 from math import pi
-
+from gpaw.pw.lfc import ft
 import numpy as np
+
 try:
     from scipy.special import spherical_jn
 
@@ -21,23 +22,6 @@ from gpaw.spherical_harmonics import Y
 def two_phi_planewave_integrals(k_Gv, setup=None, Gstart=0, Gend=None,
                                 rgd=None, phi_jg=None,
                                 phit_jg=None, l_j=None):
-    r"""Calculate PAW-correction matrix elements with planewaves.
-
-    ::
-
-      /  _       _   ik.r     _     ~   _   ik.r ~   _
-      | dr [phi (r) e    phi (r) - phi (r) e    phi (r)]
-      /        1            2         1            2
-
-                     ll   -  /     2                      ~       ~
-      = 4pi \sum_lm i  Y (k) | dr r  [ phi (r) phi (r) - phi (r) phi (r) j (kr)
-                        lm   /            1       2         1       2     ll
-
-          /
-        * | d\Omega Y     Y     Y
-          /          l1m1  l2m2  lm
-
-    """
 
     if Gend is None:
         Gend = len(k_Gv)
@@ -61,10 +45,6 @@ def two_phi_planewave_integrals(k_Gv, setup=None, Gstart=0, Gend=None,
         assert phi_jg is not None
         assert l_j is not None
 
-    ng = rgd.N
-    r_g = rgd.r_g
-    dr_g = rgd.dr_g
-
     # Construct L (l**2 + m) and j (nl) index
     L_i = []
     j_i = []
@@ -74,62 +54,46 @@ def two_phi_planewave_integrals(k_Gv, setup=None, Gstart=0, Gend=None,
             j_i.append(j)
     ni = len(L_i)
     nj = len(l_j)
-    lmax = max(l_j) * 2 + 1
+
+    if setup is not None:
+        assert ni == setup.ni and nj == setup.nj
 
     if setup is not None:
         assert ni == setup.ni and nj == setup.nj
 
     # Initialize
     npw = k_Gv.shape[0]
-    R_jj = np.zeros((nj, nj))
-    R_ii = np.zeros((ni, ni))
     phi_Gii = np.zeros((npw, ni, ni), dtype=complex)
-    j_lg = np.zeros((lmax, ng))
-
-    # Store (phi_j1 * phi_j2 - phit_j1 * phit_j2 ) for further use
-    tmp_jjg = np.zeros((nj, nj, ng))
-    for j1 in range(nj):
-        for j2 in range(nj):
-            tmp_jjg[j1, j2] = (phi_jg[j1] * phi_jg[j2] -
-                               phit_jg[j1] * phit_jg[j2])
 
     G_LLL = gaunt(max(l_j))
+    k_G = np.sum(k_Gv**2, axis=1)**0.5
 
-    # Loop over G vectors
-    for iG in range(Gstart, Gend):
-        kk = k_Gv[iG]
-        k = np.sqrt(np.dot(kk, kk))  # calculate length of q+G
+    i1_start = 0
 
-        # Calculating spherical bessel function
-        for g, r in enumerate(r_g):
-            j_lg[:, g] = sphj(lmax, k * r)
+    for j1, l1 in enumerate(l_j):
+        i2_start = 0
+        for j2, l2 in enumerate(l_j):
+            # Calculate the radial part of the product density
+            rhot_g = phi_jg[j1] * phi_jg[j2] - phit_jg[j1] * phit_jg[j2]
+            
+            for l in range((l1 + l2) % 2, l1 + l2 + 1, 2):
+                spline = rgd.spline(rhot_g, l=l, points=2**10)
+                splineG = ft(spline, N=2**12)
+                f_G = splineG.map(k_G)
 
-        for li in range(lmax):
-            # Radial part
-            for j1 in range(nj):
-                for j2 in range(nj):
-                    R_jj[j1, j2] = np.dot(r_g**2 * dr_g,
-                                          tmp_jjg[j1, j2] * j_lg[li])
-
-            for mi in range(2 * li + 1):
-                # Angular part
-                for i1 in range(ni):
-                    L1 = L_i[i1]
-                    j1 = j_i[i1]
-                    for i2 in range(ni):
-                        L2 = L_i[i2]
-                        j2 = j_i[i2]
-                        R_ii[i1, i2] = G_LLL[L1, L2, li**2 + mi] * R_jj[j1, j2]
-
-                if np.allclose(k, 0.):  # Avoid division by zero
-                    k = 1.
-                phi_Gii[iG] += R_ii * Y(li**2 + mi,
-                                        kk[0] / k,
-                                        kk[1] / k,
-                                        kk[2] / k) * (-1j)**li
-
-    phi_Gii *= 4 * pi
+                for m1 in range(2 * l1 + 1):
+                    for m2 in range(2 * l2 + 1):
+                        i1 = i1_start + m1
+                        i2 = i2_start + m2
+                        G_m = G_LLL[l1**2 + m1, l2**2 + m2, l**2:(l + 1)**2]
+                        for m, G in enumerate(G_m):
+                            if G == 0:
+                                continue
+                            x_G = Y(l**2 + m, *k_Gv.T) * f_G * (-1j)**l
+                            phi_Gii[:, i1, i2] += G * x_G
 
+            i2_start += 2 * l2 + 1
+        i1_start += 2 * l1 + 1
     return phi_Gii.reshape(npw, ni * ni)
 
 
