@@ -10,12 +10,13 @@ from gpaw.utilities import convert_string_to_fd
 from ase.utils.timing import Timer, timer
 
 import gpaw.mpi as mpi
-from gpaw.blacs import BlacsGrid, BlacsDescriptor, Redistributor
 from gpaw.response.kspair import get_calc
-from gpaw.response.kslrf import FrequencyDescriptor
+from gpaw.response.frequencies import FrequencyDescriptor
 from gpaw.response.chiks import ChiKS
 from gpaw.response.kxc import get_fxc
 from gpaw.response.kernels import get_coulomb_kernel
+from gpaw.response.pw_parallelization import Blocks1D
+from gpaw.response.groundstate import ResponseGroundStateAdapter
 
 
 class FourComponentSusceptibilityTensor:
@@ -23,9 +24,8 @@ class FourComponentSusceptibilityTensor:
 
     def __init__(self, gs, fxc='ALDA', fxckwargs={},
                  eta=0.2, ecut=50, gammacentered=False,
-                 disable_point_group=True, disable_time_reversal=True,
-                 bandsummation='pairwise', nbands=None,
-                 bundle_integrals=True, bundle_kptpairs=False,
+                 disable_point_group=False, disable_time_reversal=False,
+                 bandsummation='pairwise', nbands=None, bundle_integrals=True,
                  world=mpi.world, nblocks=1, txt=sys.stdout):
         """
         Currently, everything is in plane wave mode.
@@ -39,8 +39,7 @@ class FourComponentSusceptibilityTensor:
         eta, ecut, gammacentered
         disable_point_group,
         disable_time_reversal,
-        bandsummation, nbands,
-        bundle_integrals, bundle_kptpairs,
+        bandsummation, nbands, bundle_integrals,
         world, nblocks, txt : see gpaw.response.chiks, gpaw.response.kslrf
         """
         # Initiate output file and timer
@@ -51,6 +50,7 @@ class FourComponentSusceptibilityTensor:
 
         # Load ground state calculation
         self.calc = get_calc(gs, fd=self.fd, timer=self.timer)
+        self.gs = ResponseGroundStateAdapter(self.calc)
 
         # The plane wave basis is defined by keywords
         self.ecut = None if ecut is None else ecut / Hartree
@@ -63,17 +63,15 @@ class FourComponentSusceptibilityTensor:
                            disable_time_reversal=disable_time_reversal,
                            bandsummation=bandsummation, nbands=nbands,
                            bundle_integrals=bundle_integrals,
-                           bundle_kptpairs=bundle_kptpairs, world=world,
-                           nblocks=nblocks, txt=self.fd, timer=self.timer)
+                           world=world, nblocks=nblocks, txt=self.fd,
+                           timer=self.timer)
         self.fxc = get_fxc(self.calc, fxc,
                            response='susceptibility', mode='pw',
                            world=self.chiks.world, txt=self.chiks.fd,
                            timer=self.timer, **fxckwargs)
 
         # Parallelization over frequencies depends on the frequency input
-        self.mynw = None
-        self.w1 = None
-        self.w2 = None
+        self.blocks1d = None
 
     def get_macroscopic_component(self, spincomponent, q_c, frequencies,
                                   filename=None, txt=None):
@@ -141,7 +139,7 @@ class FourComponentSusceptibilityTensor:
         chi_w = chi_wGG[:, 0, 0]
 
         # Collect data for all frequencies
-        omega_w = wd.get_data() * Hartree
+        omega_w = wd.omega_w * Hartree
         chiks_w = self.collect(chiks_w)
         chi_w = self.collect(chi_w)
 
@@ -174,7 +172,7 @@ class FourComponentSusceptibilityTensor:
 
         if filename is None:
             tup = (spincomponent,
-                   *tuple((q_c * self.calc.wfs.kd.N_c).round()))
+                   *tuple((q_c * self.gs.kd.N_c).round()))
             filename = 'chi%sGG_q«%+d-%+d-%+d».pckl' % tup
 
         (omega_w, G_Gc, chiks_wGG,
@@ -216,7 +214,7 @@ class FourComponentSusceptibilityTensor:
                                                         frequencies, txt=txt)
 
         # Get frequencies in eV
-        omega_w = wd.get_data() * Hartree
+        omega_w = wd.omega_w * Hartree
 
         # Get susceptibility in a reduced plane wave representation
         mask_G = get_pw_reduction_map(pd, array_ecut)
@@ -251,7 +249,6 @@ class FourComponentSusceptibilityTensor:
         chi_wGG : ndarray
             The process' block of the full susceptibility component
         """
-        pd = self.get_PWDescriptor(q_c)
 
         # Initiate new call-output file, if supplied
         if txt is not None:
@@ -262,56 +259,57 @@ class FourComponentSusceptibilityTensor:
                 self.cfd.close()
             # Initiate new output file
             self.cfd = convert_string_to_fd(txt, self.world)
-        # Print to output file
-        if str(self.fd) != str(self.cfd) or txt is not None:
+        # Print to output file(s)
+        if str(self.fd) != str(self.cfd):
             print('---------------', file=self.fd)
             print(f'Calculating susceptibility spincomponent={spincomponent}'
-                  f'with q_c={pd.kd.bzk_kc[0]}', file=self.fd)
+                  f'with q_c={q_c}', flush=True, file=self.fd)
+        if txt is not None:
+            print('---------------', file=self.fd)
+            print(f'Calculating susceptibility spincomponent={spincomponent}'
+                  f'with q_c={q_c}', file=self.cfd)
+            print('---------------', flush=True, file=self.cfd)
 
-        wd = FrequencyDescriptor(np.asarray(frequencies) / Hartree)
+        wd = FrequencyDescriptor.from_array_or_dict(frequencies)
 
         # Initialize parallelization over frequencies
-        nw = len(wd)
-        self.mynw = (nw + self.world.size - 1) // self.world.size
-        self.w1 = min(self.mynw * self.world.rank, nw)
-        self.w2 = min(self.w1 + self.mynw, nw)
+        self.blocks1d = Blocks1D(self.world, len(wd))
 
-        return self._calculate_component(spincomponent, pd, wd)
+        return self._calculate_component(spincomponent, q_c, wd)
 
-    def get_PWDescriptor(self, q_c):
-        """Get the planewave descriptor defining the plane wave basis for the
-        given momentum transfer q_c."""
-        from gpaw.kpt_descriptor import KPointDescriptor
-        from gpaw.pw.descriptor import PWDescriptor
-        q_c = np.asarray(q_c, dtype=float)
-        qd = KPointDescriptor([q_c])
-        pd = PWDescriptor(self.ecut, self.calc.wfs.gd,
-                          complex, qd, gammacentered=self.gammacentered)
-        return pd
-
-    def _calculate_component(self, spincomponent, pd, wd):
+    def _calculate_component(self, spincomponent, q_c, wd):
         """In-place calculation of the given spin-component."""
+        pd, chiks_wGG = self.calculate_ks_component(spincomponent, q_c,
+                                                    wd, txt=self.cfd)
+
+        Kxc_GG = self.get_xc_kernel(spincomponent, pd, chiks_wGG=chiks_wGG)
         if spincomponent in ['+-', '-+']:
             # No Hartree kernel
-            K_GG = self.fxc(spincomponent, pd, txt=self.cfd)
+            assert Kxc_GG is not None
+            Khxc_GG = Kxc_GG
         else:
-            # Calculate Hartree kernel
-            Kbare_G = get_coulomb_kernel(pd, self.calc.wfs.kd.N_c)
-            vsqrt_G = Kbare_G ** 0.5
-            K_GG = np.eye(len(vsqrt_G)) * vsqrt_G * vsqrt_G[:, np.newaxis]
+            Khxc_GG = self.get_hartree_kernel(pd)
+            if Kxc_GG is not None:  # Kxc can be None in the RPA case
+                Khxc_GG += Kxc_GG
 
-            # Calculate exchange-correlation kernel
-            Kxc_GG = self.fxc(spincomponent, pd, txt=self.cfd)
-            if Kxc_GG is not None:
-                K_GG += Kxc_GG
-            
-        chiks_wGG = self.calculate_ks_component(spincomponent, pd,
-                                                wd, txt=self.cfd)
+        chi_wGG = self.invert_dyson(chiks_wGG, Khxc_GG)
 
-        chi_wGG = self.invert_dyson(chiks_wGG, K_GG)
+        print('\nFinished calculating component', file=self.cfd)
+        print('---------------', flush=True, file=self.cfd)
 
         return pd, wd, chiks_wGG, chi_wGG
 
+    def get_xc_kernel(self, spincomponent, pd, **ignored):
+        return self.fxc(spincomponent, pd, txt=self.cfd)
+
+    def get_hartree_kernel(self, pd):
+        """Calculate the Hartree kernel"""
+        Kbare_G = get_coulomb_kernel(pd, self.gs.kd.N_c)
+        vsqrt_G = Kbare_G ** 0.5
+        Kh_GG = np.eye(len(vsqrt_G)) * vsqrt_G * vsqrt_G[:, np.newaxis]
+
+        return Kh_GG
+
     def write_timer(self):
         """Write timer to call-output file and initiate a new."""
         self.timer.write(self.cfd)
@@ -324,28 +322,30 @@ class FourComponentSusceptibilityTensor:
         self.chiks.pme.timer = self.timer
         self.fxc.timer = self.timer
 
-    def calculate_ks_component(self, spincomponent, pd, wd, txt=None):
+    def calculate_ks_component(self, spincomponent, q_c, wd, txt=None):
         """Calculate a single component of the Kohn-Sham susceptibility tensor.
 
         Parameters
         ----------
-        spincomponent : see gpaw.response.chiks, gpaw.response.kslrf
-        pd, wd : see calculate_component
+        spincomponent, q_c : see gpaw.response.chiks, gpaw.response.kslrf
+        wd : see calculate_component
 
         Returns
         -------
+        pd : PWDescriptor
+            see gpaw.response.chiks, gpaw.response.kslrf
         chiks_wGG : ndarray
             The process' block of the Kohn-Sham susceptibility component
         """
         # ChiKS calculates the susceptibility distributed over plane waves
-        _, chiks_wGG = self.chiks.calculate(pd, wd, txt=txt,
-                                            spincomponent=spincomponent)
+        pd, chiks_wGG = self.chiks.calculate(q_c, wd, txt=txt,
+                                             spincomponent=spincomponent)
 
         # Redistribute memory, so each block has its own frequencies, but all
         # plane waves (for easy invertion of the Dyson-like equation)
-        chiks_wGG = self.distribute_frequencies(chiks_wGG)
+        chiks_wGG = self.chiks.distribute_frequencies(chiks_wGG)
 
-        return chiks_wGG
+        return pd, chiks_wGG
 
     @timer('Invert dyson-like equation')
     def invert_dyson(self, chiks_wGG, Khxc_GG):
@@ -353,11 +353,10 @@ class FourComponentSusceptibilityTensor:
 
         chi = chi_ks + chi_ks Khxc chi
         """
+        print('Inverting Dyson-like equation', flush=True, file=self.cfd)
         chi_wGG = np.empty_like(chiks_wGG)
         for w, chiks_GG in enumerate(chiks_wGG):
-            chi_GG = np.dot(np.linalg.inv(np.eye(len(chiks_GG)) +
-                                          np.dot(chiks_GG, Khxc_GG)),
-                            chiks_GG)
+            chi_GG = invert_dyson_single_frequency(chiks_GG, Khxc_GG)
 
             chi_wGG[w] = chi_GG
 
@@ -365,26 +364,21 @@ class FourComponentSusceptibilityTensor:
 
     def collect(self, a_w):
         """Collect frequencies from all blocks"""
-        # More documentation is needed! XXX
-        world = self.chiks.world
-        b_w = np.zeros(self.mynw, a_w.dtype)
-        b_w[:self.w2 - self.w1] = a_w
-        nw = len(self.chiks.omega_w)
-        A_w = np.empty(world.size * self.mynw, a_w.dtype)
-        world.all_gather(b_w, A_w)
-        return A_w[:nw]
+        return self.blocks1d.collect(a_w)
 
     def gather(self, A_wGG, wd):
         """Gather a full susceptibility array to root."""
         # Allocate arrays to gather (all need to be the same shape)
-        shape = (self.mynw,) + A_wGG.shape[1:]
+        blocks1d = self.blocks1d
+        shape = (blocks1d.blocksize,) + A_wGG.shape[1:]
         tmp_wGG = np.empty(shape, dtype=A_wGG.dtype)
-        tmp_wGG[:self.w2 - self.w1] = A_wGG
+        tmp_wGG[:blocks1d.nlocal] = A_wGG
 
         # Allocate array for the gathered data
         if self.world.rank == 0:
             # Make room for all frequencies
-            shape = (self.mynw * self.world.size,) + A_wGG.shape[1:]
+            Npadded = blocks1d.blocksize * blocks1d.blockcomm.size
+            shape = (Npadded,) + A_wGG.shape[1:]
             allA_wGG = np.empty(shape, dtype=A_wGG.dtype)
         else:
             allA_wGG = None
@@ -397,46 +391,6 @@ class FourComponentSusceptibilityTensor:
 
         return allA_wGG
 
-    @timer('Distribute frequencies')
-    def distribute_frequencies(self, chiks_wGG):
-        """Distribute frequencies to all cores."""
-        # More documentation is needed! XXX
-        world = self.chiks.world
-        comm = self.chiks.interblockcomm
-
-        if world.size == 1:
-            return chiks_wGG
-
-        nw = len(self.chiks.omega_w)
-        nG = chiks_wGG.shape[2]
-        mynw = (nw + world.size - 1) // world.size
-        mynG = (nG + comm.size - 1) // comm.size
-
-        wa = min(world.rank * mynw, nw)
-        wb = min(wa + mynw, nw)
-
-        if self.chiks.interblockcomm.size == 1:
-            return chiks_wGG[wa:wb].copy()
-
-        if self.chiks.intrablockcomm.rank == 0:
-            bg1 = BlacsGrid(comm, 1, comm.size)
-            in_wGG = chiks_wGG.reshape((nw, -1))
-        else:
-            bg1 = BlacsGrid(None, 1, 1)
-            # bg1 = DryRunBlacsGrid(mpi.serial_comm, 1, 1)
-            in_wGG = np.zeros((0, 0), complex)
-        md1 = BlacsDescriptor(bg1, nw, nG**2, nw, mynG * nG)
-
-        bg2 = BlacsGrid(world, world.size, 1)
-        md2 = BlacsDescriptor(bg2, nw, nG**2, mynw, nG**2)
-
-        r = Redistributor(world, md1, md2)
-        shape = (wb - wa, nG, nG)
-        out_wGG = np.empty(shape, complex)
-        r.redistribute(in_wGG, out_wGG.reshape((wb - wa, nG**2)))
-
-        return out_wGG
-
     def close(self):
         self.timer.write(self.cfd)
         print('\nClosing, %s' % ctime(), file=self.cfd)
@@ -445,6 +399,18 @@ class FourComponentSusceptibilityTensor:
         self.fd.close()
 
 
+def invert_dyson_single_frequency(chiks_GG, Khxc_GG):
+    """Invert single frequency Dyson equation in plane wave basis:
+
+    chi_GG' = chiks_GG + chiks_GG1 Khxc_G1G2 chi_G2G'
+    """
+    enhancement_GG = np.linalg.inv(np.eye(len(chiks_GG)) -
+                                   np.dot(chiks_GG, Khxc_GG))
+    chi_GG = np.dot(enhancement_GG, chiks_GG)
+
+    return chi_GG
+
+
 def get_pw_reduction_map(pd, ecut):
     """Get a mask to reduce the plane wave representation.
 
@@ -491,6 +457,50 @@ def get_pw_coordinates(pd):
     return np.round(np.dot(G_Gv, np.linalg.inv(B_cv))).astype(int)
 
 
+def get_inverted_pw_mapping(pd1, pd2):
+    """Get the plane wave coefficients mapping GG' of pd1 into -G-G' of pd2"""
+    G1_Gc = get_pw_coordinates(pd1)
+    G2_Gc = get_pw_coordinates(pd2)
+
+    mG2_G1 = []
+    for G1_c in G1_Gc:
+        found_match = False
+        for G2, G2_c in enumerate(G2_Gc):
+            if np.all(G2_c == -G1_c):
+                mG2_G1.append(G2)
+                found_match = True
+                break
+        if not found_match:
+            raise ValueError('Could not match pd1 and pd2')
+
+    # Set up mapping from GG' to -G-G'
+    invmap_GG = tuple(np.meshgrid(mG2_G1, mG2_G1, indexing='ij'))
+
+    return invmap_GG
+
+
+def symmetrize_reciprocity(pd, A_wGG):
+    """In collinear systems without spin-orbit coupling, the plane wave
+    susceptibility is reciprocal in the sense that e.g.
+
+    χ_(GG')^(+-)(q, ω) = χ_(-G'-G)^(+-)(-q, ω)
+
+    This method symmetrizes A_wGG in the case where q=0.
+    """
+    from gpaw.test.response.test_chiks import get_inverted_pw_mapping
+
+    q_c = pd.kd.bzk_kc[0]
+    if np.allclose(q_c, 0.):
+        invmap_GG = get_inverted_pw_mapping(pd, pd)
+        for A_GG in A_wGG:
+            tmp_GG = np.zeros_like(A_GG)
+
+            # Symmetrize [χ_(GG')(q, ω) + χ_(-G'-G)(-q, ω)] / 2
+            tmp_GG += A_GG
+            tmp_GG += A_GG[invmap_GG].T
+            A_GG[:] = tmp_GG / 2.
+
+
 def write_macroscopic_component(omega_w, chiks_w, chi_w, filename, world):
     """Write the spatially averaged dynamic susceptibility."""
     assert isinstance(filename, str)
@@ -504,7 +514,7 @@ def write_macroscopic_component(omega_w, chiks_w, chi_w, filename, world):
 
 def read_macroscopic_component(filename):
     """Read a stored macroscopic susceptibility file"""
-    d = np.loadtxt(filename, delimiter=', ')
+    d = np.loadtxt(filename, delimiter=',')
     omega_w = d[:, 0]
     chiks_w = np.array(d[:, 1], complex)
     chiks_w.imag = d[:, 2]
