@@ -9,7 +9,7 @@ from typing import IO, Tuple
 
 import numpy as np
 from ase.data import atomic_names, atomic_numbers
-from ase.units import Bohr, Hartree
+from ase.units import Bohr, Ha
 
 from gpaw import setup_paths
 from gpaw.atom.radialgd import (AbinitRadialGridDescriptor,
@@ -71,6 +71,7 @@ class SetupData:
         self.nct_g = None
         self.nvt_g = None
         self.vbar_g = None
+        self.vt_g = None
 
         # Kinetic energy densities of core electrons
         self.tauc_g = None
@@ -89,8 +90,10 @@ class SetupData:
 
         # Optional quantities, normally not used
         self.X_p = None
+        self.X_wp = {}
         self.X_pg = None
         self.ExxC = None
+        self.ExxC_w = {}
         self.X_gamma = None
         self.extra_xc_data = {}
         self.phicorehole_g = None
@@ -139,9 +142,9 @@ class SetupData:
 
     def print_info(self, text, setup):
         if self.phicorehole_g is None:
-            text(self.symbol + '-setup:')
+            text(self.symbol + ':')
         else:
-            text(f'{self.symbol}-setup ({self.fcorehole:.1f} core hole):')
+            text(f'{self.symbol}:  # ({self.fcorehole:.1f} core hole)')
         text('  name:', atomic_names[atomic_numbers[self.symbol]])
         text('  id:', self.fingerprint)
         text('  Z:', self.Z)
@@ -152,30 +155,28 @@ class SetupData:
             text(f'  core: {self.Nc:.1f}')
         text('  charge:', self.Z - self.Nv - self.Nc)
         if setup.HubU is not None:
-            for index in range(len(setup.HubU)):
-                text('  Hubbard U: %f eV (l=%d, scale=%s)' %
-                     (setup.HubU[index] * Hartree,
-                      setup.Hubl[index],
-                      bool(setup.Hubs[index])))
+            for U, l, scale in zip(setup.HubU, setup.Hubl, setup.Hubs):
+                text(f'  Hubbard: {{U: {U * Ha},  # eV\n'
+                     f'            l: {l},\n'
+                     f'            scale: {bool(scale)}}}')
         text('  file:', self.filename)
-        text('  compensation charges: {}, rc={:.2f}, lmax={}'
-             .format(self.shape_function['type'],
-                     self.shape_function['rc'] * Bohr,
-                     setup.lmax))
-        text(('  cutoffs: {:.2f}(filt), {:.2f}(core),'
-              .format(setup.rcutfilter * Bohr,
-                      setup.rcore * Bohr)))
+        sf = self.shape_function
+        text(f'  compensation charges: {{type: {sf["type"]},\n'
+             f'                         rc: {sf["rc"] * Bohr:.2f},\n'
+             f'                         lmax: {setup.lmax}}}')
+        text(f'  cutoffs: {{filter: {setup.rcutfilter * Bohr:.2f},\n'
+             f'            core: {setup.rcore * Bohr:.2f}}}')
         text('  valence states:')
-        text('                energy  radius')
+        text('    #              energy  rcut')
         j = 0
         for n, l, f, eps in zip(self.n_j, self.l_j, self.f_j, self.eps_j):
             if n > 0:
                 f = f'({f:.2f})'
-                text('    %d%s%-5s %9.3f   %5.3f' % (
-                    n, 'spdf'[l], f, eps * Hartree, self.rcut_j[j] * Bohr))
+                text('    - %d%s%-5s %9.3f   %5.3f' % (
+                    n, 'spdf'[l], f, eps * Ha, self.rcut_j[j] * Bohr))
             else:
-                text('    *%s       %9.3f   %5.3f' % (
-                    'spdf'[l], eps * Hartree, self.rcut_j[j] * Bohr))
+                text('    -  %s       %9.3f   %5.3f' % (
+                    'spdf'[l], eps * Ha, self.rcut_j[j] * Bohr))
             j += 1
         text()
 
@@ -345,6 +346,12 @@ class SetupData:
                     print(f'{x!r}', end=' ', file=xml)
                 print(f'\n  </{name}>', file=xml)
 
+        if self.vt_g is not None:
+            xml.write('  <pseudo_potential grid="g1">\n')
+            for x in self.vt_g:
+                print(f'{x!r}', end=' ', file=xml)
+            print('\n  </pseudo_potential>', file=xml)
+
         print('  <kinetic_energy_differences>', end=' ', file=xml)
         nj = len(self.e_kin_jj)
         for j1 in range(nj):
@@ -358,8 +365,16 @@ class SetupData:
             for x in self.X_p:
                 print(f'{x!r}', end=' ', file=xml)
             print('\n  </exact_exchange_X_matrix>', file=xml)
-
+            
             print(f'  <exact_exchange core-core="{self.ExxC!r}"/>', file=xml)
+            for omega, Ecc in self.ExxC_w.items():
+                print(f'  <erfc_exchange omega="{omega}" core-core="{Ecc}"/>',
+                      file=xml)
+                print(f'  <erfc_exchange_X_matrix omega="{omega}" X_p="',
+                      end=' ', file=xml)
+                for x in self.X_wp[omega]:
+                    print(f'{x!r}', end=' ', file=xml)
+                print('"/>', file=xml)
 
         if self.X_pg is not None:
             print('  <yukawa_exchange_X_matrix>\n    ', end=' ', file=xml)
@@ -418,7 +433,7 @@ You need to set the GPAW_SETUP_PATH environment variable to point to
 the directories where PAW dataset and basis files are stored.  See
 https://wiki.fysik.dtu.dk/gpaw/install.html#install-paw-datasets
 for details."""
-        raise RuntimeError(f'{err}\n{helpful_message}\n')
+        raise FileNotFoundError(f'{err}\n{helpful_message}\n')
 
     return filename, source
 
@@ -515,7 +530,8 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
                       'localized_potential', 'yukawa_exchange_X_matrix',
                       'kinetic_energy_differences', 'exact_exchange_X_matrix',
                       'ae_core_kinetic_energy_density',
-                      'pseudo_core_kinetic_energy_density']:
+                      'pseudo_core_kinetic_energy_density',
+                      'pseudo_potential']:
             self.data = []
         elif name.startswith('GLLB_'):
             self.data = []
@@ -525,8 +541,13 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
         elif name == 'projector_function':
             self.id = attrs['state']
             self.data = []
+        elif name == 'erfc_exchange':
+            setup.ExxC_w[float(attrs['omega'])] = float(attrs['core-core'])
         elif name == 'exact_exchange':
             setup.ExxC = float(attrs['core-core'])
+        elif name == 'erfc_exchange_X_matrix':
+            X_p = np.array([float(x) for x in ''.join(attrs['X_p']).split()])
+            setup.X_wp[float(attrs['omega'])] = X_p
         elif name == 'yukawa_exchange':
             setup.X_gamma = float(attrs['gamma'])
         elif name == 'core_hole_state':
@@ -576,6 +597,8 @@ class PAWXMLParser(xml.sax.handler.ContentHandler):
             setup.tauct_g = x_g
         elif name in ['localized_potential', 'zero_potential']:  # XXX
             setup.vbar_g = x_g
+        elif name in ['pseudo_potential']:
+            setup.vt_g = x_g
         elif name.startswith('GLLB_'):
             # Add setup tags starting with GLLB_ to extra_xc_data. Remove
             # GLLB_ from front of string:
