@@ -4,7 +4,6 @@
 # Please see the accompanying LICENSE file for further information.
 from typing import Tuple
 
-from ase.io import read
 from ase.utils import gcd
 import numpy as np
 
@@ -551,15 +550,66 @@ def atoms2symmetry(atoms, id_a=None, tolerance=1e-7):
 
 
 class CLICommand:
-    """Analyse symmetry."""
+    """Analyse symmetry (and show IBZ k-points).
+
+    Example:
+
+        $ ase build -x bcc -a 3.5 Li | gpaw symmetry -k "{density:3,gamma:1}"
+        symmetry:
+          number of symmetries: 48
+          number of symmetries with translation: 0
+
+        bz sampling:
+          number of bz points: 512
+          number of ibz points: 29
+          monkhorst-pack size: [8, 8, 8]
+          monkhorst-pack shift: [0.0625, 0.0625, 0.0625]
+    """
 
     @staticmethod
     def add_arguments(parser):
-        parser.add_argument('--tolerance', type=float, default=1e-7)
-        parser.add_argument('filename')
+        parser.add_argument('-t', '--tolerance', type=float, default=1e-7,
+                            help='Tolerance used for identifying symmetries.')
+        parser.add_argument(
+            '-k', '--k-points',
+            help='Use symmetries to reduce number of k-points.  '
+            'Exapmples: "4,4,4", "{density:3.5,gamma:True}".')
+        parser.add_argument('-v', '--verbose', action='store_true',
+                            help='Show symmetry operations (and k-points).')
+        parser.add_argument('-s', '--symmorphic', action='store_true',
+                            help='Only find symmorphic symmetries.')
+        parser.add_argument('filename', nargs='?', default='-',
+                            help='Filename to read structure from.  '
+                            'Use "-" for reading from stdin.  '
+                            'Default is "-".')
 
     @staticmethod
     def run(args):
-        atoms = read(args.filename)
-        symmetry = atoms2symmetry(atoms, toletance=args.tolerance)
-        print(symmetry)
+        import sys
+        from gpaw.new.symmetry import create_symmetries_object
+        from gpaw.new.builder import create_kpts
+        from gpaw.new.input_parameters import kpts
+        from ase.cli.run import str2dict
+        from ase.db import connect
+        from ase.io import read
+
+        if args.filename == '-':
+            atoms = next(connect(sys.stdin).select()).toatoms()
+        else:
+            atoms = read(args.filename)
+        symmetries = create_symmetries_object(
+            atoms,
+            parameters={'tolerance': args.tolerance,
+                        'symmorphic': args.symmorphic})
+        txt = str(symmetries)
+        if not args.verbose:
+            txt = txt.split('  rotations', 1)[0]
+        print(txt)
+        if args.k_points:
+            k = str2dict('kpts=' + args.k_points)['kpts']
+            bz = create_kpts(kpts(k), atoms)
+            ibz = symmetries.reduce(bz)
+            txt = str(ibz)
+            if not args.verbose:
+                txt = txt.split('  points and weights:', 1)[0]
+            print(txt)
