@@ -1,4 +1,5 @@
 # Written by Lauri Lehtovaara 2008
+import numpy as np
 
 from gpaw.utilities.blas import axpy
 
@@ -8,32 +9,29 @@ class MultiBlas:
         self.gd = gd
 
     # Multivector ZAXPY: a x + y => y
-    def multi_zaxpy(self, a,x,y, nvec):
+    def multi_zaxpy(self, a, x, y, nvec):
         if isinstance(a, (float, complex)):
             for i in range(nvec):
-                axpy(a*(1+0J), x[i], y[i])
+                axpy(a * (1 + 0j), x[i], y[i])
         else:
             for i in range(nvec):
-                axpy(a[i]*(1.0+0.0J), x[i], y[i])
+                axpy(a[i] * (1.0 + 0.0j), x[i], y[i])
 
     # Multivector dot product, a^H b, where ^H is transpose
-    def multi_zdotc(self, s, x,y, nvec):
+    def multi_zdotc(self, s, x, y, nvec):
         for i in range(nvec):
-            s[i] = np.vdot(x[i],y[i])
+            s[i] = np.vdot(x[i], y[i])
         self.gd.comm.sum(s)
         return s
 
     # Multiscale: a x => x
-    def multi_scale(self, a,x, nvec):
+    def multi_scale(self, a, x, nvec):
         if isinstance(a, (float, complex)):
             x *= a
         else:
             for i in range(nvec):
                 x[i] *= a[i]
 
-# -------------------------------------------------------------------
-
-import numpy as np
 
 class BandPropertyMonitor:
     def __init__(self, wfs, name, interval=1):
@@ -49,7 +47,7 @@ class BandPropertyMonitor:
         self.niter += self.interval
 
     def update(self, wfs):
-        #strictly serial XXX!
+        # strictly serial XXX!
         data_un = []
 
         for u, kpt in enumerate(wfs.kpt_u):
@@ -62,10 +60,11 @@ class BandPropertyMonitor:
     def write(self, data):
         pass
 
+
 class BandPropertyWriter(BandPropertyMonitor):
     def __init__(self, filename, wfs, name, interval=1):
         BandPropertyMonitor.__init__(self, wfs, name, interval)
-        self.fileobj = open(filename,'w')
+        self.fileobj = open(filename, 'w')
 
     def write(self, data):
         self.fileobj.write(data.tostring())
@@ -74,8 +73,6 @@ class BandPropertyWriter(BandPropertyMonitor):
     def __del__(self):
         self.fileobj.close()
 
-# -------------------------------------------------------------------
-
 
 class StaticOverlapMonitor:
     def __init__(self, wfs, wf_u, P_aui, interval=1):
@@ -92,20 +89,19 @@ class StaticOverlapMonitor:
         self.niter += self.interval
 
     def update(self, wfs, calculate_P_ani=False):
-
-        #strictly serial XXX!
+        # strictly serial XXX!
         Porb_un = []
 
         for u, kpt in enumerate(wfs.kpt_u):
             swf = self.wf_u[u].ravel()
 
-            psit_n = kpt.psit_nG.reshape((len(kpt.f_n),-1))
+            psit_n = kpt.psit_nG.reshape((len(kpt.f_n), -1))
             Porb_n = np.dot(psit_n.conj(), swf) * wfs.gd.dv
 
             P_ani = kpt.P_ani
 
             if calculate_P_ani:
-                #wfs.pt.integrate(psit_nG, P_ani, kpt.q)
+                # wfs.pt.integrate(psit_nG, P_ani, kpt.q)
                 raise NotImplementedError(
                     'In case you were wondering, TODO XXX')
 
@@ -115,8 +111,8 @@ class StaticOverlapMonitor:
                     for i in range(len(P_ni[0])):
                         for j in range(len(P_ni[0])):
                             Porb_n[n] += (P_ni[n][i].conj() *
-                                       wfs.setups[a].dO_ii[i][j] *
-                                       sP_i[j])
+                                          wfs.setups[a].dO_ii[i][j] *
+                                          sP_i[j])
 
             Porb_un.append(Porb_n)
 
@@ -125,10 +121,11 @@ class StaticOverlapMonitor:
     def write(self, data):
         pass
 
+
 class StaticOverlapWriter(StaticOverlapMonitor):
     def __init__(self, filename, wfs, overlap, interval=1):
         StaticOverlapMonitor.__init__(self, wfs, overlap, interval)
-        self.fileobj = open(filename,'w')
+        self.fileobj = open(filename, 'w')
 
     def write(self, data):
         self.fileobj.write(data.tostring())
@@ -137,8 +134,6 @@ class StaticOverlapWriter(StaticOverlapMonitor):
     def __del__(self):
         self.fileobj.close()
 
-# -------------------------------------------------------------------
-
 
 class DynamicOverlapMonitor:
     def __init__(self, wfs, overlap, interval=1):
@@ -155,7 +150,7 @@ class DynamicOverlapMonitor:
 
     def update(self, wfs, calculate_P_ani=False):
 
-        #strictly serial XXX!
+        # strictly serial XXX!
         S_unn = []
 
         for kpt in wfs.kpt_u:
@@ -163,12 +158,14 @@ class DynamicOverlapMonitor:
             P_ani = kpt.P_ani
 
             if calculate_P_ani:
-                #wfs.pt.integrate(psit_nG, P_ani, kpt.q)
+                # wfs.pt.integrate(psit_nG, P_ani, kpt.q)
                 raise NotImplementedError(
                     'In case you were wondering, TODO XXX')
 
             # Construct the overlap matrix:
-            S = lambda x: x
+            def S(x):
+                return x
+
             dS_aii = dict([(a, self.setups[a].dO_ii) for a in P_ani])
             S_nn = self.operator.calculate_matrix_elements(psit_nG, P_ani,
                                                            S, dS_aii)
@@ -179,10 +176,11 @@ class DynamicOverlapMonitor:
     def write(self, data):
         pass
 
+
 class DynamicOverlapWriter(DynamicOverlapMonitor):
     def __init__(self, filename, wfs, overlap, interval=1):
         DynamicOverlapMonitor.__init__(self, wfs, overlap, interval)
-        self.fileobj = open(filename,'w')
+        self.fileobj = open(filename, 'w')
 
     def write(self, data):
         self.fileobj.write(data.tostring())
