@@ -14,5 +14,5 @@ def test_fileio_idiotproof_setup(in_tmp_dir):
     m.calc = calc
     m.get_potential_energy()
     calc.write('r.gpw')
-    calc = GPAW('r.gpw', xc='PBE', idiotproof=False)
+    calc = GPAW('r.gpw', xc='PBE')
     calc.get_potential_energy()
