@@ -1,5 +1,5 @@
 from ase.build import molecule
-from gpaw import GPAW, Mixer, Davidson
+from gpaw import GPAW, MixerSum, Davidson
 
 # Move atom infinitesimally across cell border and test that SCF loop is still
 # well converged afterwards.  If it is /not/ well converged, then the code
@@ -31,7 +31,8 @@ def test_generic_move_across_cell():
         assert calc.scf.niter == 3
 
     def kwargs():
-        return dict(xc='oldLDA', mixer=Mixer(0.7), kpts=[1, 1, 2])
+        # Make sure MixerSum works for spin-paired system also:
+        return dict(xc='oldLDA', mixer=MixerSum(0.7), kpts=[1, 1, 2])
 
     test(GPAW(mode='lcao', basis='sz(dzp)', h=0.3))
     test(GPAW(mode='pw', eigensolver=Davidson(3),
