@@ -3,16 +3,14 @@ from ase import Atoms, Atom
 from gpaw import GPAW
 from gpaw.atom.generator import Generator
 from gpaw.atom.configurations import parameters
-from gpaw import setup_paths
 from gpaw.test import equal
 from gpaw.mpi import world
 
 
 @pytest.mark.gllb
 @pytest.mark.libxc
-def test_gllb_ne(in_tmp_dir):
+def test_gllb_ne(in_tmp_dir, add_cwd_to_setup_paths):
     atom = 'Ne'
-    setup_paths.insert(0, '.')
 
     for xcname in ['GLLBSC', 'GLLB']:
         if world.rank == 0:
