@@ -37,7 +37,7 @@ def test_rraman_albrecht():
 
     # check size
     kss = KSSingles('rraman-d0.010.eq.ex.gz')
-    assert(len(kss) == 1)
+    assert len(kss) == 1
 
     om = 5
 
