@@ -75,7 +75,7 @@ def test_lrtddft_kssingles_Be(in_tmp_dir):
             mpi.world.barrier()
             kss = KSSingles.read(fname)
             kss1 = KSSingles.read(fname, restrict={'jend': 1})
-            assert(len(kss1) == calc.wfs.kd.nibzkpts * calc.wfs.nspins)
+            assert len(kss1) == calc.wfs.kd.nibzkpts * calc.wfs.nspins
 
         # periodic and non-periodic should be roughly equal
         equal(energy['zero_bc'], energy['periodic'], 5.e-2)
