@@ -18,14 +18,6 @@ a = 4.0
 c = 5.0
 
 
-class GPAW_with_classmethod_read(GPAW):
-    @classmethod
-    def read(cls, filename):
-        gpw = cls()
-        GPAW.read(gpw, filename)
-        return gpw
-
-
 def test_lrtddft_placzek_profeta_albrecht(in_tmp_dir):
     from ase.vibrations.albrecht import Albrecht
     from ase.vibrations.placzek import Placzek, Profeta
@@ -38,26 +30,25 @@ def test_lrtddft_placzek_profeta_albrecht(in_tmp_dir):
     name = exname = 'rraman'
     exkwargs = {'restrict': {'eps': 0.0, 'jend': 3}}
 
-    if 1:
-        calc = GPAW_with_classmethod_read(
-            xc=xc, nbands=7,
-            convergence={'bands': 3},
-            spinpol=False,
-            # eigensolver='rmm-diis',
-            symmetry={'point_group': False},
-            txt=txt)
-        H2.calc = calc
-        # H2.get_potential_energy()
+    calc = GPAW(
+        xc=xc, nbands=7,
+        convergence={'bands': 3},
+        spinpol=False,
+        # eigensolver='rmm-diis',
+        symmetry={'point_group': False},
+        txt=txt)
+    H2.calc = calc
+    # H2.get_potential_energy()
 
-        rr = ResonantRamanCalculator(
-            H2, KSSingles, name=name, exname=exname,
-            exkwargs=exkwargs,
-            # XXX full does not work in parallel due to boxes
-            # on different nodes
-            # overlap=lambda x, y: Overlap(x).full(y)[0],
-            overlap=lambda x, y: Overlap(x).pseudo(y)[0],
-            txt=txt)
-        rr.run()
+    rr = ResonantRamanCalculator(
+        H2, KSSingles, name=name, exname=exname,
+        exkwargs=exkwargs,
+        # XXX full does not work in parallel due to boxes
+        # on different nodes
+        # overlap=lambda x, y: Overlap(x).full(y)[0],
+        overlap=lambda x, y: Overlap(x).pseudo(y)[0],
+        txt=txt)
+    rr.run()
 
     # check
 
