@@ -15,19 +15,19 @@ def test_digonalize():
     atoms.get_potential_energy()
 
     lr = LrTDDFT(atoms.calc)
-    
+
     # all
     lr.diagonalize()
-    assert(len(lr) == 10)
+    assert len(lr) == 10
 
     lr.diagonalize(restrict={'istart': 3})
-    assert(len(lr) == 1)
-    
+    assert len(lr) == 1
+
     lr.diagonalize(restrict={'jend': 1})
-    assert(len(lr) == 1)
-    
+    assert len(lr) == 1
+
     lr.diagonalize(restrict={'eps': 1.5})
-    assert(len(lr) == 2)
-    
+    assert len(lr) == 2
+
     lr.diagonalize(restrict={'energy_range': 1})
-    assert(len(lr) == 3)
+    assert len(lr) == 3
