@@ -1,11 +1,9 @@
-import pytest
 from ase import Atoms
 from gpaw.calculator import GPAW as OldGPAW
 from gpaw.new.ase_interface import GPAW as NewGPAW
 
 
-@pytest.mark.serial
-def test_refactored_code():
+def test_refactored_code(in_tmp_dir):
     new('n')
 
 
@@ -13,7 +11,8 @@ def new(x):
     params = {'mode': {'name': 'fd', 'force_complex_dtype': 0},
               'random': not True,
               'kpts': (4, 1, 1),
-              'spinpol': not True}
+              'spinpol': not True,
+              'convergence': {'maximum iterations': 200}}
 
     if x == 'n':
         GPAW = NewGPAW
@@ -26,6 +25,7 @@ def new(x):
                       txt=f'{x}s.txt')
     f = atoms.get_forces()
     e = atoms.get_potential_energy()
+    atoms.get_dipole_moment()
     print(e)
     print(f)
     if 1:
