@@ -6,7 +6,8 @@ from ase.build import molecule
 from gpaw import GPAW
 from gpaw.tddft import TDDFT as GRIDTDDFT
 from gpaw.lcaotddft import LCAOTDDFT
-from gpaw.poisson import PoissonSolver
+from gpaw.poisson import PoissonSolver as PS
+from gpaw.poisson_moment import MomentCorrectionPoissonSolver
 from gpaw.poisson_extravacuum import ExtraVacuumPoissonSolver
 
 pytestmark = pytest.mark.skipif(world.size > 2,
@@ -15,20 +16,33 @@ pytestmark = pytest.mark.skipif(world.size > 2,
 
 def test_poisson_poisson_restart(in_tmp_dir):
     name = 'Na2'
-    poissoneps = 1e-16
     gpts = np.array([16, 16, 24])
     # Uncomment the following line if you want to run the test with 4 cpus
     # gpts *= 2
 
-    def PS(**kwargs):
-        return PoissonSolver(eps=poissoneps, **kwargs)
-
     poissonsolver_i = []
 
     ps = PS()
     poissonsolver_i.append(ps)
 
-    ps = PS(remove_moment=4)
+    ps = MomentCorrectionPoissonSolver(poissonsolver=PS(),
+                                       moment_corrections=4)
+    poissonsolver_i.append(ps)
+
+    mom_corr_i = [{'moms': range(4), 'center': [16, 16, 4]},
+                  {'moms': range(4), 'center': [16, 16, 20]}]
+    ps = MomentCorrectionPoissonSolver(poissonsolver=PS(),
+                                       moment_corrections=4)
+    poissonsolver_i.append(ps)
+
+    ps = MomentCorrectionPoissonSolver(poissonsolver=PS(),
+                                       moment_corrections=mom_corr_i)
+    poissonsolver_i.append(ps)
+
+    ps1 = MomentCorrectionPoissonSolver(poissonsolver=PS(),
+                                        moment_corrections=mom_corr_i[0:1])
+    ps = MomentCorrectionPoissonSolver(poissonsolver=ps1,
+                                       moment_corrections=mom_corr_i[1:2])
     poissonsolver_i.append(ps)
 
     ps = ExtraVacuumPoissonSolver(gpts * 2, PS())
