@@ -1,15 +1,12 @@
-import pytest
-from gpaw.utilities import compiled_with_sl
+import os
+
 from ase import Atoms
+
 from gpaw import GPAW
-from gpaw.mpi import world, serial_comm
-
-pytestmark = pytest.mark.skipif(
-    world.size != 1 and not compiled_with_sl(),
-    reason='world.size != 1 and not compiled_with_sl()')
+from gpaw.mpi import serial_comm, world
 
 
-def test_pw_fulldiagk(in_tmp_dir):
+def test_pw_fulldiagk(in_tmp_dir, scalapack):
     a = Atoms('H',
               cell=(1, 3, 3),
               pbc=1)
@@ -51,6 +48,7 @@ def test_pw_fulldiagk(in_tmp_dir):
 
     for w in [w2, w3, w4]:
         err = abs(abs(w[1, 2, 3]) - abs(w1[1, 2, 3]))
+        print(err)
         assert err < 1e-7, err
 
     for e in [e2, e3, e4]:
@@ -59,6 +57,9 @@ def test_pw_fulldiagk(in_tmp_dir):
         err = abs(e[-1] - e2[-1])
         assert err < 1e-10, err
 
+    if os.environ.get('GPAW_NEW'):
+        return
+
     a.calc = GPAW(mode='pw',
                   h=0.15,
                   kpts=(4, 1, 1),
