@@ -1,21 +1,16 @@
 import pytest
-from gpaw.mpi import world
-from gpaw.utilities import compiled_with_sl
 import numpy as np
 from ase import Atoms
 from ase.lattice.hexagonal import Hexagonal
-from gpaw import GPAW, FermiDirac
+from gpaw import GPAW, FermiDirac, PW
 from gpaw.test import findpeak, equal
 from gpaw.response.bse import BSE
 
-pytestmark = pytest.mark.skipif(
-    world.size < 4 or not compiled_with_sl(),
-    reason='world.size < 4 or not compiled_with_sl()')
 
-
-def test_response_bse_MoS2_cut(in_tmp_dir):
+@pytest.mark.response
+def test_response_bse_MoS2_cut(in_tmp_dir, scalapack):
     if 1:
-        calc = GPAW(mode='pw',
+        calc = GPAW(mode=PW(180),
                     xc='PBE',
                     nbands='nao',
                     setups={'Mo': '6'},
@@ -29,7 +24,7 @@ def test_response_bse_MoS2_cut(in_tmp_dir):
 
         cell = Hexagonal(symbol='Mo',
                          latticeconstant={'a': a, 'c': c}).get_cell()
-        layer = Atoms(symbols='MoS2', cell=cell, pbc=(1, 1, 1),
+        layer = Atoms(symbols='MoS2', cell=cell, pbc=(1, 1, 0),
                       scaled_positions=[(0, 0, 0),
                                         (2 / 3, 1 / 3, 0.3),
                                         (2 / 3, 1 / 3, -0.3)])
@@ -38,6 +33,7 @@ def test_response_bse_MoS2_cut(in_tmp_dir):
         pos[1][2] = pos[0][2] + 3.172 / 2
         pos[2][2] = pos[0][2] - 3.172 / 2
         layer.set_positions(pos)
+        layer.center(axis=2)
         layer.calc = calc
         layer.get_potential_energy()
         calc.write('MoS2.gpw', mode='all')
@@ -56,7 +52,6 @@ def test_response_bse_MoS2_cut(in_tmp_dir):
               truncation='2D')
 
     w_w, alpha_w = bse.get_polarizability(filename=None,
-                                          pbc=[True, True, False],
                                           write_eig=None,
                                           eta=0.02,
                                           w_w=np.linspace(0., 5., 5001))
