@@ -2,8 +2,6 @@
 It takes ~109 s on one core"""
 
 import pytest
-from gpaw.mpi import world
-from gpaw.utilities import compiled_with_sl
 import numpy as np
 
 from ase.lattice.hexagonal import Graphene
@@ -12,12 +10,9 @@ from gpaw import GPAW, FermiDirac
 from gpaw.test import equal
 from gpaw.response.g0w0 import G0W0
 
-pytestmark = pytest.mark.skipif(
-    world.size != 1 and not compiled_with_sl(),
-    reason='world.size != 1 and not compiled_with_sl()')
 
-
-def test_response_gw_hBN_extrapolate(in_tmp_dir):
+@pytest.mark.response
+def test_response_gw_hBN_extrapolate(in_tmp_dir, scalapack):
     if 1:
         calc = GPAW(mode='pw',
                     xc='PBE',
@@ -48,9 +43,11 @@ def test_response_gw_hBN_extrapolate(in_tmp_dir):
     gw = G0W0('hBN.gpw',
               'gw-hBN',
               ecut=50,
-              domega0=0.1,
+              frequencies={'type': 'nonlinear',
+                           'domega0': 0.1},
               eta=0.2,
               truncation='2D',
+              q0_correction=True,
               kpts=[0],
               bands=(3, 5),
               ecut_extrapolation=[30, 40, 50],
@@ -58,7 +55,7 @@ def test_response_gw_hBN_extrapolate(in_tmp_dir):
 
     e_qp = gw.calculate()['qp'][0, 0]
 
-    ev = -4.381
-    ec = 3.71013806
+    ev = -3.9041
+    ec = 3.1858
     equal(e_qp[0], ev, 0.01)
     equal(e_qp[1], ec, 0.01)
