@@ -1,5 +1,6 @@
 import os
 
+import pytest
 import numpy as np
 
 from ase import Atoms
@@ -14,6 +15,7 @@ from gpaw.response.df import DielectricFunction
 # physical sodium cell.
 
 
+@pytest.mark.response
 def test_response_na_plasmon(in_tmp_dir):
     a = 4.23 / 2.0
     a1 = Atoms('Na',
@@ -27,18 +29,14 @@ def test_response_na_plasmon(in_tmp_dir):
                cell=(2 * a, a, a),
                pbc=True)
 
-    a1.calc = GPAW(gpts=(10, 10, 10),
-                   experimental={'niter_fixdensity': 2},
-                   mode=PW(300),
+    a1.calc = GPAW(mode=PW(250),
                    kpts={'size': (8, 8, 8), 'gamma': True},
                    parallel={'band': 1},
                    # txt='small.txt',
                    )
 
     # Kpoint sampling should be halved in the expanded direction.
-    a2.calc = GPAW(gpts=(20, 10, 10),
-                   experimental={'niter_fixdensity': 2},
-                   mode=PW(300),
+    a2.calc = GPAW(mode=PW(250),
                    kpts={'size': (4, 8, 8), 'gamma': True},
                    parallel={'band': 1},
                    # txt='large.txt',
@@ -80,10 +78,9 @@ def test_response_na_plasmon(in_tmp_dir):
             pass
 
         df1 = DielectricFunction('gs_Na_small.gpw',
-                                 domega0=0.03,
-                                 omegamax=10,
-                                 ecut=150,
+                                 ecut=40,
                                  name='chi0',
+                                 rate=0.001,
                                  **kwargs)
 
         df1NLFCx, df1LFCx = df1.get_dielectric_function(direction='x')
@@ -96,10 +93,9 @@ def test_response_na_plasmon(in_tmp_dir):
             pass
 
         df2 = DielectricFunction('gs_Na_large.gpw',
-                                 domega0=0.03,
-                                 omegamax=10,
-                                 ecut=150,
+                                 ecut=40,
                                  name='chi1',
+                                 rate=0.001,
                                  **kwargs)
 
         df2NLFCx, df2LFCx = df2.get_dielectric_function(direction='x')
@@ -114,7 +110,7 @@ def test_response_na_plasmon(in_tmp_dir):
         dfs5.append(df1LFCz)
 
         # Compare plasmon frequencies and intensities
-        w_w = df1.chi0.omega_w
+        w_w = df1.chi0.wd.omega_w
         w1, I1 = findpeak(w_w, -(1. / df1LFCx).imag)
         w2, I2 = findpeak(w_w, -(1. / df2LFCx).imag)
         equal(w1, w2, 1e-2)
