@@ -1,9 +1,10 @@
 from itertools import product
 
+import pytest
 import numpy as np
 
 from gpaw.response.integrators import TetrahedronIntegrator
-from gpaw.response.chi0 import ArrayDescriptor
+from gpaw.response.chi0 import FrequencyGridDescriptor
 # from gpaw.test import equal
 
 
@@ -15,6 +16,7 @@ def unit_sphere(x_c):
     return np.array([(x_c**2).sum()**0.5], float)
 
 
+@pytest.mark.response
 def test_tetrahedron_integrator():
     cell_cv = np.eye(3)
     integrator = TetrahedronIntegrator(cell_cv)
@@ -26,7 +28,7 @@ def test_tetrahedron_integrator():
     integrator.integrate(kind='spectral function',
                          domain=domain,
                          integrand=(unit, unit_sphere),
-                         x=ArrayDescriptor([-1.0]),
+                         x=FrequencyGridDescriptor([-1.0]),
                          out_wxx=out_wxx)
 
     assert abs(out_wxx[0, 0, 0] - 4 * np.pi) < 1e-2
