@@ -1,3 +1,4 @@
+import pytest
 from ase.build import bulk
 from gpaw import GPAW, FermiDirac, PW
 from gpaw.mpi import serial_comm
@@ -5,6 +6,7 @@ from gpaw.xc.rpa import RPACorrelation
 from gpaw.test import equal
 
 
+@pytest.mark.response
 def test_rpa_rpa_energy_Na(in_tmp_dir):
     blk = bulk('Na', 'bcc', a=4.23)
 
