@@ -1,3 +1,4 @@
+import pytest
 from ase.build import bulk
 from gpaw import GPAW, FermiDirac
 from gpaw.mpi import serial_comm
@@ -6,6 +7,7 @@ from gpaw.xc.rpa import RPACorrelation
 from gpaw.xc.fxc import FXCCorrelation
 
 
+@pytest.mark.response
 def test_rpa_rpa_energy_Ni(in_tmp_dir):
     Ni = bulk('Ni', 'fcc')
     Ni.set_initial_magnetic_moments([0.7])
