@@ -1,10 +1,11 @@
+import pytest
 from ase.build import bulk
 from gpaw import GPAW, FermiDirac
 from gpaw.mpi import serial_comm
-from gpaw.test import equal
 from gpaw.xc.rpa import RPACorrelation
 
 
+@pytest.mark.response
 def test_rpa_rpa_energy_Si(in_tmp_dir):
     a0 = 5.43
     Si = bulk('Si', a=a0)
@@ -24,5 +25,5 @@ def test_rpa_rpa_energy_Si(in_tmp_dir):
     rpa = RPACorrelation(calc, qsym=True, nfrequencies=8)
     E_rpa_qsym = rpa.calculate(ecut=[ecut])
 
-    print(E_rpa_qsym, E_rpa_noqsym, 0.001)
-    equal(E_rpa_qsym, -12.61, 0.01)
+    assert E_rpa_qsym == pytest.approx(-12.61, abs=0.01)
+    assert E_rpa_qsym == pytest.approx(E_rpa_noqsym)
