@@ -3,10 +3,12 @@ from gpaw import GPAW, restart
 
 from gpaw.test import equal
 
+
 def test_sic_scfsic_h2(in_tmp_dir):
     a = 6.0
     atom = Atoms('H', magmoms=[1.0], cell=(a, a, a))
-    molecule = Atoms('H2', positions=[(0, 0, 0), (0, 0, 0.737)], cell=(a, a, a))
+    molecule = Atoms('H2', positions=[
+                     (0, 0, 0), (0, 0, 0.737)], cell=(a, a, a))
     atom.center()
     molecule.center()
 
@@ -16,12 +18,12 @@ def test_sic_scfsic_h2(in_tmp_dir):
                 setups='hgh')
 
     atom.calc = calc
-    e1 = atom.get_potential_energy()
+    atom.get_potential_energy()
 
     molecule.calc = calc
     e2 = molecule.get_potential_energy()
-    F_ac = molecule.get_forces()
-    de = 2 * e1 - e2
+    molecule.get_forces()
+    # de = 2 * e1 - e2
     # equal(de, 4.5, 0.1)
 
     # Test forces ...
