@@ -8,7 +8,10 @@ def test_solvation_nan_radius():
     atoms = Atoms('H')
     atoms.center(vacuum=3.0)
     kwargs = solv.get_HW14_water_kwargs()
-    rfun = lambda a: [np.nan]
+
+    def rfun(a):
+        return [np.nan]
+
     kwargs['cavity'].effective_potential.atomic_radii = rfun
     atoms.calc = solv.SolvationGPAW(xc='LDA', h=0.24, **kwargs)
     try:
