@@ -29,8 +29,8 @@ def test_symmetry_fractional_translations(in_tmp_dir):
     atoms.calc = calc
     energy_fractrans = atoms.get_potential_energy()
 
-    assert(len(calc.wfs.kd.ibzk_kc) == 6)
-    assert(len(calc.wfs.kd.symmetry.op_scc) == 16)
+    assert len(calc.wfs.kd.ibzk_kc) == 6
+    assert len(calc.wfs.kd.symmetry.op_scc) == 16
 
     # without fractional translations
     calc = GPAW(mode=PW(),
@@ -44,7 +44,7 @@ def test_symmetry_fractional_translations(in_tmp_dir):
     atoms.calc = calc
     energy_no_fractrans = atoms.get_potential_energy()
 
-    assert(len(calc.wfs.kd.ibzk_kc) == 8)
-    assert(len(calc.wfs.kd.symmetry.op_scc) == 8)
+    assert len(calc.wfs.kd.ibzk_kc) == 8
+    assert len(calc.wfs.kd.symmetry.op_scc) == 8
 
     equal(energy_fractrans, energy_no_fractrans, 1e-5)
