@@ -1,8 +1,12 @@
+import os
+
 import pytest
 from ase import Atoms
+
 from gpaw import GPAW
 
 
+@pytest.mark.ci
 def test_fixdensity(in_tmp_dir):
     a = 2.5
     slab = Atoms('Li', cell=(a, a, 2 * a), pbc=1)
@@ -39,6 +43,9 @@ def test_fixdensity(in_tmp_dir):
     assert e2 == pytest.approx(e1, abs=3e-5)
     assert e3 == pytest.approx(e1, abs=3e-5)
 
+    if os.environ.get('GPAW_NEW'):
+        return
+
     calc = GPAW('li.gpw',
                 txt='li-4.txt',
                 fixdensity=True,
