@@ -14,12 +14,10 @@ def test_xc_nonselfconsistent(in_tmp_dir):
     calc = GPAW(gpts=(n, n, n), nbands=1, xc=xc('PBE'))
     atoms.calc = calc
     e1 = atoms.get_potential_energy()
-    niter1 = calc.get_number_of_iterations()
     e1ref = calc.get_reference_energy()
     de12 = calc.get_xc_difference(xc('revPBE'))
     calc.set(xc=xc('revPBE'))
     e2 = atoms.get_potential_energy()
-    niter2 = calc.get_number_of_iterations()
     e2ref = calc.get_reference_energy()
     de21 = calc.get_xc_difference(xc('PBE'))
     print(e1ref + e1 + de12 - (e2ref + e2))
@@ -34,6 +32,5 @@ def test_xc_nonselfconsistent(in_tmp_dir):
     equal(de21, de21b, 9e-8)
 
     energy_tolerance = 0.0005
-    niter_tolerance = 0
     equal(e1, -0.07904951, energy_tolerance)
     equal(e2, -0.08147563, energy_tolerance)
