@@ -17,18 +17,15 @@ def test_xc_revPBE_Li():
                 occupations=FermiDirac(0.0))
     li.calc = calc
     e = li.get_potential_energy() + calc.get_reference_energy()
-    niter_PBE = calc.get_number_of_iterations()
     equal(e, -7.462 * Hartree, 1.4)
 
     calc.set(xc=dict(name='revPBE', stencil=1))
     erev = li.get_potential_energy() + calc.get_reference_energy()
-    niter_revPBE = calc.get_number_of_iterations()
 
     equal(erev, -7.487 * Hartree, 1.3)
     equal(e - erev, 0.025 * Hartree, 0.002 * Hartree)
 
     print(e, erev)
     energy_tolerance = 0.002
-    niter_tolerance = 0
     equal(e, -204.381098849, energy_tolerance)  # svnversion 5252
     equal(erev, -205.012303379, energy_tolerance)  # svnversion 5252
