@@ -29,14 +29,14 @@ class WannierOverlaps:
 
         self.atoms = atoms
         self.nwannier = nwannier
-        self.monkhorst_pack_size = np.asarray(monkhorst_pack_size)
+        self.monkhorst_pack_size = np.array(monkhorst_pack_size)
         self.kpoints = kpoints
         self.fermi_level = fermi_level
         self.directions = directions
 
         self.nkpts, ndirs, self.nbands, nbands = overlaps.shape
         assert nbands == self.nbands
-        assert self.nkpts == np.prod(monkhorst_pack_size)
+        assert self.nkpts == np.prod(monkhorst_pack_size)  # type: ignore
         assert ndirs == len(directions)
 
         self._overlaps = overlaps
