@@ -6,7 +6,7 @@ from ase.units import Ha
 from gpaw.occupations import OccupationNumberCalculator
 from gpaw.projections import Projections
 from gpaw.utilities import pack, unpack2
-from gpaw.utilities.blas import gemm, axpy
+from gpaw.utilities.blas import mmm, axpy
 from gpaw.utilities.partition import AtomPartition
 
 
@@ -142,8 +142,8 @@ class WaveFunctions:
             P_Mi = self.P_aqMi[a][kpt.q]
             rhoP_Mi = np.zeros_like(P_Mi)
             D_ii = np.zeros(D_sii[kpt.s].shape, kpt.rho_MM.dtype)
-            gemm(1.0, P_Mi, kpt.rho_MM, 0.0, rhoP_Mi)
-            gemm(1.0, rhoP_Mi, P_Mi.T.conj().copy(), 0.0, D_ii)
+            mmm(1.0, kpt.rho_MM, 'N', P_Mi, 'N', 0.0, rhoP_Mi)
+            mmm(1.0, P_Mi, 'C', rhoP_Mi, 'N', 0.0, D_ii)
             D_sii[kpt.s] += D_ii.real
         else:
             if self.collinear:
