@@ -12,26 +12,28 @@ from gpaw.hybrids.paw import pawexxvv
 from gpaw.xc import XC
 from gpaw.xc.tools import vxc
 from gpaw.xc.kernel import XCNull
-from gpaw.response.pair import PairDensity
+from gpaw.response.pair import NoCalculatorPairDensity
 from gpaw.pw.descriptor import PWDescriptor
 from gpaw.kpt_descriptor import KPointDescriptor
 from gpaw.utilities import unpack, unpack2
 from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
+from gpaw.response.context import new_context
 
 
-def select_kpts(kpts, calc):
+def select_kpts(kpts, kd):
     """Function to process input parameters that take a list of k-points given
     in different format and returns a list of indices of the corresponding
     k-points in the IBZ."""
+
     if kpts is None:
         # Do all k-points in the IBZ:
-        return np.arange(calc.wfs.kd.nibzkpts)
+        return np.arange(kd.nibzkpts)
 
     if np.asarray(kpts).ndim == 1:
         return kpts
 
     # Find k-points:
-    bzk_Kc = calc.get_bz_k_points()
+    bzk_Kc = kd.bzk_kc
     indices = []
     for k_c in kpts:
         d_Kc = bzk_Kc - k_c
@@ -40,13 +42,13 @@ def select_kpts(kpts, calc):
         if not np.allclose(d_Kc[K], 0):
             raise ValueError('Could not find k-point: {k_c}'
                              .format(k_c=k_c))
-        k = calc.wfs.kd.bz2ibz_k[K]
+        k = kd.bz2ibz_k[K]
         indices.append(k)
     return indices
 
 
-class EXX(PairDensity):
-    def __init__(self, calc, xc=None, kpts=None, bands=None, ecut=None,
+class EXX(NoCalculatorPairDensity):
+    def __init__(self, gs, xc=None, kpts=None, bands=None, ecut=None,
                  stencil=2,
                  omega=None, world=mpi.world, txt=sys.stdout, timer=None):
         """Non self-consistent hybrid functional calculations.
@@ -61,7 +63,7 @@ class EXX(PairDensity):
         kpts: list of in or list of list of int
             List of indices of the IBZ k-points to calculate the quasi particle
             energies for.  Default is all k-points.  One can also specify the
-            coordiantes of the k-point.  As an example, Gamma and X for an
+            coordinates of the k-point.  As an example, Gamma and X for an
             FCC crystal would be: kpts=[[0, 0, 0], [1 / 2, 1 / 2, 0]].
         bands: tuple of two ints
             Range of band indices, like (n1, n2), to calculate the quasi
@@ -73,8 +75,9 @@ class EXX(PairDensity):
             for the ground-state calculations.
         """
 
-        PairDensity.__init__(self, calc, ecut, world=world, txt=txt,
-                             timer=timer)
+        context = new_context(txt=txt, world=world, timer=timer)
+
+        super().__init__(gs=gs, context=context)
 
         def _xc(name):
             return {'name': name, 'stencil': stencil}
@@ -101,7 +104,7 @@ class EXX(PairDensity):
         self.omega = omega
         self.exc = np.nan  # density dependent part of xc-energy
 
-        self.kpts = select_kpts(kpts, self.calc)
+        self.kpts = select_kpts(kpts, self.gs.kd)
 
         if bands is None:
             # Do all occupied bands:
@@ -114,12 +117,14 @@ class EXX(PairDensity):
 
         self.bands = bands
 
-        if self.ecut is None:
-            self.ecut = self.calc.wfs.pd.ecut
+        if ecut is None:
+            ecut = self.gs.pd.ecut
+        self.ecut = ecut
+
         print('Plane-wave cutoff: %.3f eV' % (self.ecut * Hartree),
               file=self.fd)
 
-        shape = (self.calc.wfs.nspins, len(self.kpts), bands[1] - bands[0])
+        shape = (self.gs.nspins, len(self.kpts), bands[1] - bands[0])
         self.exxvv_sin = np.zeros(shape)   # valence-valence exchange energies
         self.exxvc_sin = np.zeros(shape)   # valence-core exchange energies
         self.f_sin = np.empty(shape)       # occupation numbers
@@ -131,18 +136,14 @@ class EXX(PairDensity):
         self.exxvc = np.nan  # valence-core
         self.exxcc = 0.0     # core-core
 
-        self.mysKn1n2 = None  # my (s, K, n1, n2) indices
-        self.distribute_k_points_and_bands(0, self.nocc2)
-
-        # All occupied states:
-        self.mykpts = [self.get_k_point(s, K, n1, n2)
-                       for s, K, n1, n2 in self.mysKn1n2]
+        self.pair_distribution = self.distribute_k_points_and_bands(
+            0, self.nocc2)
 
         if omega is None:
             print('Using Wigner-Seitz truncated coulomb interaction.',
                   file=self.fd)
-            self.wstc = WignerSeitzTruncatedCoulomb(self.calc.wfs.gd.cell_cv,
-                                                    self.calc.wfs.kd.N_c,
+            self.wstc = WignerSeitzTruncatedCoulomb(self.gs.gd.cell_cv,
+                                                    self.gs.kd.N_c,
                                                     self.fd)
         self.iG_qG = {}  # cache
 
@@ -158,8 +159,8 @@ class EXX(PairDensity):
             Name of restart json-file.  Allows for incremental calculation
             of eigenvalues.
         """
-        kd = self.calc.wfs.kd
-        nspins = self.calc.wfs.nspins
+        kd = self.gs.kd
+        nspins = self.gs.nspins
 
         if restart:
             if isinstance(restart, str):
@@ -191,7 +192,7 @@ class EXX(PairDensity):
                 K1 = kd.ibz2bz_k[k1]
                 kpt1 = self.get_k_point(s, K1, *self.bands)
                 self.f_sin[s, i] = kpt1.f_n
-                for kpt2 in self.mykpts:
+                for kpt2 in self.pair_distribution.mykpts:
                     if kpt2.s == s:
                         self.calculate_q(i, kpt1, kpt2)
 
@@ -240,22 +241,23 @@ class EXX(PairDensity):
         return self.exx * Hartree
 
     def get_total_energy(self):
-        ham = self.calc.hamiltonian
+        ham = self.gs.hamiltonian
         return (self.exx * self.exx_fraction + self.exc +
                 ham.e_total_free - ham.e_xc) * Hartree
 
     def get_eigenvalue_contributions(self):
         b1, b2 = self.bands
-        e_sin = vxc(self.calc, self.xc)[:, self.kpts, b1:b2] / Hartree
+        e_sin = vxc(
+            self.gs, self.xc)[:, self.kpts, b1:b2] / Hartree
         e_sin += (self.exxvv_sin + self.exxvc_sin) * self.exx_fraction
         return e_sin * Hartree
 
     def calculate_q(self, i, kpt1, kpt2):
-        wfs = self.calc.wfs
+        gs = self.gs
 
-        q_c = wfs.kd.bzk_kc[kpt2.K] - wfs.kd.bzk_kc[kpt1.K]
+        q_c = gs.kd.bzk_kc[kpt2.K] - gs.kd.bzk_kc[kpt1.K]
         qd = KPointDescriptor([q_c])
-        pd = PWDescriptor(self.ecut, wfs.gd, wfs.dtype, kd=qd)
+        pd = PWDescriptor(self.ecut, gs.gd, gs.dtype, kd=qd)
 
         Q_G = self.get_fft_indices(kpt1.K, kpt2.K, q_c, pd,
                                    kpt1.shift_c - kpt2.shift_c)
@@ -273,7 +275,7 @@ class EXX(PairDensity):
 
     def calculate_n(self, pd, n_mG, kpt2):
         iG_G = self.get_coulomb_kernel(pd)
-        x = 4 * pi / self.calc.wfs.kd.nbzkpts / pd.gd.dv**2
+        x = 4 * pi / self.gs.kd.nbzkpts / pd.gd.dv**2
 
         e = 0.0
         for f, n_G in zip(kpt2.f_n, n_mG):
@@ -295,7 +297,7 @@ class EXX(PairDensity):
                         G2_G[1:])**0.5
             return iG_G
 
-        key = tuple((pd.kd.bzk_kc[0] * self.calc.wfs.kd.N_c).round())
+        key = tuple((pd.kd.bzk_kc[0] * self.gs.kd.N_c).round())
         iG_G = self.iG_qG.get(key)
         if iG_G is None:
             v_G = self.wstc.get_potential(pd)
@@ -304,11 +306,11 @@ class EXX(PairDensity):
         return iG_G
 
     def initialize_paw_exx_corrections(self):
-        for a, atomdata in enumerate(self.calc.wfs.setups):
+        for a, atomdata in enumerate(self.gs.setups):
             V_sii = []
-            for D_p in self.calc.density.D_asp[a]:
+            for D_p in self.gs.D_asp[a]:
                 D_ii = unpack2(D_p)
-                V_ii = pawexxvv(atomdata, D_ii)
+                V_ii = pawexxvv(atomdata.M_pp, D_ii)
                 V_sii.append(V_ii)
             if atomdata.X_p is None:
                 C_ii = D_ii * 0.0
@@ -319,7 +321,7 @@ class EXX(PairDensity):
             self.exxcc += atomdata.ExxC or 0.0
 
     def calculate_paw_exx_corrections(self, i, kpt):
-        x = self.calc.wfs.nspins / self.world.size
+        x = self.gs.nspins / self.world.size
         s = kpt.s
 
         for V_sii, C_ii, P_ni in zip(self.V_asii, self.C_aii, kpt.P_ani):
@@ -330,7 +332,7 @@ class EXX(PairDensity):
             self.exxvc_sin[s, i] -= c_n
 
     def calculate_hybrid_correction(self):
-        dens = self.calc.density
+        dens = self.gs.density
         if dens.nt_sg is None:
             dens.interpolate_pseudo_density()
         exc = self.xc.calculate(dens.finegd, dens.nt_sg)
