@@ -5,16 +5,15 @@ from time import ctime
 import numpy as np
 from ase.units import Hartree
 from ase.utils import IOContext
-from ase.utils.timing import timer, Timer
-from scipy.special.orthogonal import p_roots
+from ase.utils.timing import Timer, timer
+from scipy.special import p_roots
 
 import gpaw.mpi as mpi
 from gpaw import GPAW
-from gpaw.kpt_descriptor import KPointDescriptor
 from gpaw.response.chi0 import Chi0
 from gpaw.response.kernels import get_coulomb_kernel
 from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
-from gpaw.pw.descriptor import PWDescriptor, count_reciprocal_vectors
+from gpaw.response.groundstate import ResponseGroundStateAdapter
 
 
 def rpa(filename, ecut=200.0, blocks=1, extrapolate=4):
@@ -87,11 +86,12 @@ class RPACorrelation:
             energy from different q-points
         """
 
+        self.iocontext = IOContext()
         if isinstance(calc, str):
             calc = GPAW(calc, txt=None, communicator=mpi.serial_comm)
         self.calc = calc
+        self.gs = ResponseGroundStateAdapter(calc)
 
-        self.iocontext = IOContext()
         self.fd = self.iocontext.openfile(txt, world)
 
         self.timer = Timer()
@@ -131,7 +131,7 @@ class RPACorrelation:
         self.iocontext.close()
 
     def initialize_q_points(self, qsym):
-        kd = self.calc.wfs.kd
+        kd = self.gs.kd
         self.bzq_qc = kd.get_bz_q_points(first=True)
 
         if not qsym:
@@ -179,7 +179,7 @@ class RPACorrelation:
         """Calculate RPA correlation energy for one or several cutoffs.
 
         ecut: float or list of floats
-            Plane-wave cutoff(s).
+            Plane-wave cutoff(s) in eV.
         nbands: int
             Number of bands (defaults to number of plane-waves).
         spin: bool
@@ -210,33 +210,24 @@ class RPACorrelation:
             self.read()
             self.world.barrier()
 
-        chi0 = Chi0(self.calc, frequencies=1j * Hartree * self.omega_w,
-                    eta=0.0, intraband=False, hilbert=False,
-                    txt='chi0.txt', timer=self.timer, world=self.world,
-                    nblocks=self.nblocks)
+        chi0calc = Chi0(
+            self.calc, frequencies=1j * Hartree * self.omega_w,
+            eta=0.0, intraband=False, hilbert=False,
+            txt='chi0.txt', timer=self.timer, world=self.world,
+            nblocks=self.nblocks,
+            ecut=ecutmax * Hartree)
 
-        self.blockcomm = chi0.blockcomm
+        self.blockcomm = chi0calc.blockcomm
 
-        wfs = self.calc.wfs
+        gs = self.gs
 
         if self.truncation == 'wigner-seitz':
-            self.wstc = WignerSeitzTruncatedCoulomb(wfs.gd.cell_cv,
-                                                    wfs.kd.N_c, self.fd)
+            self.wstc = WignerSeitzTruncatedCoulomb(gs.gd.cell_cv,
+                                                    gs.kd.N_c, self.fd)
         else:
             self.wstc = None
 
         nq = len(self.energy_qi)
-        nw = len(self.omega_w)
-        nGmax = max(count_reciprocal_vectors(ecutmax, wfs.gd, q_c)
-                    for q_c in self.ibzq_qc[nq:])
-        mynGmax = (nGmax + self.nblocks - 1) // self.nblocks
-
-        nx = (1 + spin) * nw * mynGmax * nGmax
-        A1_x = np.empty(nx, complex)
-        if self.nblocks > 1:
-            A2_x = np.empty(nx, complex)
-        else:
-            A2_x = None
 
         self.timer.start('RPA')
 
@@ -248,26 +239,15 @@ class RPACorrelation:
                 p()
                 continue
 
-            thisqd = KPointDescriptor([q_c])
-            pd = PWDescriptor(ecutmax, wfs.gd, complex, thisqd)
+            chi0_s = [chi0calc.create_chi0(q_c, extend_head=False)]
+            if spin:
+                chi0_s.append(chi0calc.create_chi0(q_c, extend_head=False))
+
+            pd = chi0_s[0].pd
             nG = pd.ngmax
-            mynG = (nG + self.nblocks - 1) // self.nblocks
-            chi0.Ga = self.blockcomm.rank * mynG
-            chi0.Gb = min(chi0.Ga + mynG, nG)
-
-            shape = (1 + spin, nw, chi0.Gb - chi0.Ga, nG)
-            chi0_swGG = A1_x[:np.prod(shape)].reshape(shape)
-            chi0_swGG[:] = 0.0
-
-            if np.allclose(q_c, 0.0):
-                chi0_swxvG = np.zeros((1 + spin, nw, 2, 3, nG), complex)
-                chi0_swvv = np.zeros((1 + spin, nw, 3, 3), complex)
-            else:
-                chi0_swxvG = None
-                chi0_swvv = None
 
             # First not completely filled band:
-            m1 = chi0.nocc1
+            m1 = chi0calc.nocc1
             p('# %s  -  %s' % (len(self.energy_qi), ctime().split()[-2]))
             p('q = [%1.3f %1.3f %1.3f]' % tuple(q_c))
 
@@ -284,18 +264,19 @@ class RPACorrelation:
                 p('E_cut = %d eV / Bands = %d:' % (ecut * Hartree, m2))
                 self.fd.flush()
 
-                energy = self.calculate_q(chi0, pd,
-                                          chi0_swGG, chi0_swxvG, chi0_swvv,
-                                          m1, m2, cut_G, A2_x)
+                energy = self.calculate_q(chi0calc,
+                                          chi0_s,
+                                          m1, m2, cut_G)
 
                 energy_i.append(energy)
                 m1 = m2
 
-                a = 1 / chi0.kncomm.size
+                a = 1 / chi0calc.kncomm.size
                 if ecut < ecutmax and a != 1.0:
                     # Chi0 will be summed again over chicomm, so we divide
                     # by its size:
-                    chi0_swGG *= a
+                    for chi0 in chi0_s:
+                        chi0.chi0_wGG *= a
                     # if chi0_swxvG is not None:
                     #     chi0_swxvG *= a
                     #     chi0_swvv *= a
@@ -327,29 +308,25 @@ class RPACorrelation:
         return e_i * Hartree
 
     @timer('chi0(q)')
-    def calculate_q(self, chi0, pd, chi0_swGG, chi0_swxvG, chi0_swvv,
-                    m1, m2, cut_G, A2_x):
-        chi0_wGG = chi0_swGG[0]
-        if chi0_swxvG is not None:
-            chi0_wxvG = chi0_swxvG[0]
-            chi0_wvv = chi0_swvv[0]
-        else:
-            chi0_wxvG = None
-            chi0_wvv = None
-        chi0._calculate(pd, chi0_wGG, chi0_wxvG, chi0_wvv,
-                        m1, m2, spins='all', extend_head=False)
+    def calculate_q(self, chi0calc, chi0_s,
+                    m1, m2, cut_G):
+        chi0 = chi0_s[0]
+        chi0calc.update_chi0(chi0,
+                             m1, m2, spins='all')
 
         print('E_c(q) = ', end='', file=self.fd)
 
-        chi0_wGG = chi0.redistribute(chi0_wGG, A2_x)
+        chi0_wGG = chi0.redistribute()
 
-        if not pd.kd.gamma:
-            e = self.calculate_energy(pd, chi0_wGG, cut_G)
+        kd = self.gs.kd
+        if not chi0.pd.kd.gamma:
+            e = self.calculate_energy(chi0.pd, chi0_wGG, cut_G)
             print('%.3f eV' % (e * Hartree), file=self.fd)
             self.fd.flush()
         else:
             from ase.dft import monkhorst_pack
-            kd = self.calc.wfs.kd
+            # XXXX again a redundant implementation of the thing
+            # now in gpaw.response.gamma_int !
             N = 4
             N_c = np.array([N, N, N])
             if self.truncation is not None:
@@ -359,22 +336,23 @@ class RPACorrelation:
             U_scc = kd.symmetry.op_scc
             q_qc = kd.get_ibz_q_points(q_qc, U_scc)[0]
             weight_q = kd.q_weights
-            q_qv = 2 * np.pi * np.dot(q_qc, pd.gd.icell_cv)
+            q_qv = 2 * np.pi * np.dot(q_qc, chi0.pd.gd.icell_cv)
 
             nw = len(self.omega_w)
             mynw = nw // self.nblocks
             w1 = self.blockcomm.rank * mynw
             w2 = w1 + mynw
-            a_qw = np.sum(np.dot(chi0_wvv[w1:w2], q_qv.T) * q_qv.T, axis=1).T
-            a0_qwG = np.dot(q_qv, chi0_wxvG[w1:w2, 0])
-            a1_qwG = np.dot(q_qv, chi0_wxvG[w1:w2, 1])
+            a_qw = np.sum(np.dot(chi0.chi0_wvv[w1:w2], q_qv.T) * q_qv.T,
+                          axis=1).T
+            a0_qwG = np.dot(q_qv, chi0.chi0_wxvG[w1:w2, 0])
+            a1_qwG = np.dot(q_qv, chi0.chi0_wxvG[w1:w2, 1])
 
             e = 0
             for iq in range(len(q_qv)):
                 chi0_wGG[:, 0] = a0_qwG[iq]
                 chi0_wGG[:, :, 0] = a1_qwG[iq]
                 chi0_wGG[:, 0, 0] = a_qw[iq]
-                ev = self.calculate_energy(pd, chi0_wGG, cut_G,
+                ev = self.calculate_energy(chi0.pd, chi0_wGG, cut_G,
                                            q_v=q_qv[iq])
                 e += ev * weight_q[iq]
             print('%.3f eV' % (e * Hartree), file=self.fd)
@@ -386,7 +364,7 @@ class RPACorrelation:
     def calculate_energy(self, pd, chi0_wGG, cut_G, q_v=None):
         """Evaluate correlation energy from chi0."""
 
-        sqrV_G = get_coulomb_kernel(pd, self.calc.wfs.kd.N_c, q_v=q_v,
+        sqrV_G = get_coulomb_kernel(pd, self.gs.kd.N_c, q_v=q_v,
                                     truncation=self.truncation,
                                     wstc=self.wstc)**0.5
         if cut_G is not None:
@@ -434,13 +412,13 @@ class RPACorrelation:
         p('Started at:  ', ctime())
         p()
         p('Atoms                          :',
-          self.calc.atoms.get_chemical_formula(mode='hill'))
-        p('Ground state XC functional     :', self.calc.hamiltonian.xc.name)
-        p('Valence electrons              :', self.calc.wfs.setups.nvalence)
-        p('Number of bands                :', self.calc.wfs.bd.nbands)
-        p('Number of spins                :', self.calc.wfs.nspins)
-        p('Number of k-points             :', len(self.calc.wfs.kd.bzk_kc))
-        p('Number of irreducible k-points :', len(self.calc.wfs.kd.ibzk_kc))
+          self.gs.atoms.get_chemical_formula(mode='hill'))
+        p('Ground state XC functional     :', self.gs.xcname)
+        p('Valence electrons              :', self.gs.nvalence)
+        p('Number of bands                :', self.gs.bd.nbands)
+        p('Number of spins                :', self.gs.nspins)
+        p('Number of k-points             :', len(self.gs.kd.bzk_kc))
+        p('Number of irreducible k-points :', len(self.gs.kd.ibzk_kc))
         p('Number of q-points             :', len(self.bzq_qc))
         p('Number of irreducible q-points :', len(self.ibzq_qc))
         p()
